@@ -67,6 +67,8 @@ typedef struct _compiler_ctx_t {
     statement_ctx_t *stat_ctx;
     function_code_t *func;
 
+    unsigned loc;
+
     function_expression_t *func_head;
     function_expression_t *func_tail;
 
@@ -208,6 +210,11 @@ static BSTR compiler_alloc_bstr_len(compiler_ctx_t *ctx, const WCHAR *str, size_
     return ctx->code->bstr_pool[ctx->code->bstr_cnt++];
 }
 
+void set_compiler_loc(compiler_ctx_t *ctx, unsigned loc)
+{
+    ctx->loc = loc;
+}
+
 static unsigned push_instr(compiler_ctx_t *ctx, jsop_t op)
 {
     assert(ctx->code_size >= ctx->code_off);
@@ -224,6 +231,7 @@ static unsigned push_instr(compiler_ctx_t *ctx, jsop_t op)
     }
 
     ctx->code->instrs[ctx->code_off].op = op;
+    ctx->code->instrs[ctx->code_off].loc = ctx->loc;
     return ctx->code_off++;
 }
 
@@ -1245,6 +1253,7 @@ static HRESULT compile_while_statement(compiler_ctx_t *ctx, while_statement_t *s
     if(FAILED(hres))
         return hres;
 
+    set_compiler_loc(ctx, stat->stat.loc);
     if(stat->do_while) {
         label_set_addr(ctx, stat_ctx.continue_label);
         hres = compile_expression(ctx, stat->expr, TRUE);
@@ -1292,6 +1301,7 @@ static HRESULT compile_for_statement(compiler_ctx_t *ctx, for_statement_t *stat)
     expr_off = ctx->code_off;
 
     if(stat->expr) {
+        set_compiler_loc(ctx, stat->expr_loc);
         hres = compile_expression(ctx, stat->expr, TRUE);
         if(FAILED(hres))
             return hres;
@@ -1308,6 +1318,7 @@ static HRESULT compile_for_statement(compiler_ctx_t *ctx, for_statement_t *stat)
     label_set_addr(ctx, stat_ctx.continue_label);
 
     if(stat->end_expr) {
+        set_compiler_loc(ctx, stat->end_loc);
         hres = compile_expression(ctx, stat->end_expr, FALSE);
         if(FAILED(hres))
             return hres;
@@ -1613,6 +1624,7 @@ static HRESULT compile_switch_statement(compiler_ctx_t *ctx, switch_statement_t
             continue;
         }
 
+        set_compiler_loc(ctx, iter->loc);
         hres = compile_expression(ctx, iter->expr, TRUE);
         if(FAILED(hres))
             break;
@@ -1751,6 +1763,7 @@ static HRESULT compile_try_statement(compiler_ctx_t *ctx, try_statement_t *stat)
         if(FAILED(hres))
             return hres;
 
+        set_compiler_loc(ctx, stat->finally_loc);
         if(!push_instr(ctx, OP_end_finally))
             return E_OUTOFMEMORY;
     }
@@ -1772,6 +1785,8 @@ static HRESULT compile_statement(compiler_ctx_t *ctx, statement_ctx_t *stat_ctx,
         ctx->stat_ctx = stat_ctx;
     }
 
+    set_compiler_loc(ctx, stat->loc);
+
     switch(stat->type) {
     case STAT_BLOCK:
         hres = compile_block_statement(ctx, ((block_statement_t*)stat)->stat_list);
@@ -2243,20 +2258,30 @@ void release_bytecode(bytecode_t *code)
     heap_free(code);
 }
 
-static HRESULT init_code(compiler_ctx_t *compiler, const WCHAR *source)
+static HRESULT init_code(compiler_ctx_t *compiler, const WCHAR *source, UINT64 source_context, unsigned start_line)
 {
+    size_t len = source ? lstrlenW(source) : 0;
+
+    if(len > INT32_MAX)
+        return E_OUTOFMEMORY;
+
     compiler->code = heap_alloc_zero(sizeof(bytecode_t));
     if(!compiler->code)
         return E_OUTOFMEMORY;
 
     compiler->code->ref = 1;
+    compiler->code->source_context = source_context;
+    compiler->code->start_line = start_line;
     heap_pool_init(&compiler->code->heap);
 
-    compiler->code->source = heap_strdupW(source);
+    compiler->code->source = heap_alloc((len + 1) * sizeof(WCHAR));
     if(!compiler->code->source) {
         release_bytecode(compiler->code);
         return E_OUTOFMEMORY;
     }
+    if(len)
+        memcpy(compiler->code->source, source, len * sizeof(WCHAR));
+    compiler->code->source[len] = 0;
 
     compiler->code->instrs = heap_alloc(64 * sizeof(instr_t));
     if(!compiler->code->instrs) {
@@ -2279,6 +2304,7 @@ static HRESULT compile_function(compiler_ctx_t *ctx, source_elements_t *source,
 
     TRACE("\n");
 
+    func->bytecode = ctx->code;
     ctx->func_head = ctx->func_tail = NULL;
     ctx->from_eval = from_eval;
     ctx->func = func;
@@ -2458,13 +2484,13 @@ static HRESULT compile_arguments(compiler_ctx_t *ctx, const WCHAR *args)
     return parse_arguments(ctx, args, ctx->code->global_code.params, NULL);
 }
 
-HRESULT compile_script(script_ctx_t *ctx, const WCHAR *code, const WCHAR *args, const WCHAR *delimiter,
-        BOOL from_eval, BOOL use_decode, bytecode_t **ret)
+HRESULT compile_script(script_ctx_t *ctx, const WCHAR *code, UINT64 source_context, unsigned start_line,
+                       const WCHAR *args, const WCHAR *delimiter, BOOL from_eval, BOOL use_decode, bytecode_t **ret)
 {
     compiler_ctx_t compiler = {0};
     HRESULT hres;
 
-    hres = init_code(&compiler, code);
+    hres = init_code(&compiler, code, source_context, start_line);
     if(FAILED(hres))
         return hres;
 
