@@ -245,6 +245,17 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetThreadId( HANDLE thread )
 }
 
 
+/***********************************************************************
+ *           GetThreadIdealProcessorEx   (kernelbase.@)
+ */
+BOOL WINAPI /* DECLSPEC_HOTPATCH */ GetThreadIdealProcessorEx( HANDLE thread, PROCESSOR_NUMBER *ideal )
+{
+    FIXME( "(%p %p): stub\n", thread, ideal );
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
+
+
 /***********************************************************************
  *	GetThreadLocale   (kernelbase.@)
  */
@@ -404,8 +415,8 @@ HRESULT WINAPI DECLSPEC_HOTPATCH SetThreadDescription( HANDLE thread, PCWSTR des
     if (length > USHRT_MAX)
         return HRESULT_FROM_NT(STATUS_INVALID_PARAMETER);
 
-    info.Length = length << 16 | length;
-    info.Description = (WCHAR *)description;
+    info.Description.Length = info.Description.MaximumLength = length;
+    info.Description.Buffer = (WCHAR *)description;
 
     return HRESULT_FROM_NT(NtSetInformationThread( thread, ThreadDescription, &info, sizeof(info) ));
 }
@@ -434,15 +445,13 @@ HRESULT WINAPI DECLSPEC_HOTPATCH GetThreadDescription( HANDLE thread, WCHAR **de
     status = NtQueryInformationThread( thread, ThreadDescription, info, length, &length );
     if (!status)
     {
-        length = info->Length & 0xffff;
-
-        if (!(*description = LocalAlloc( 0, length + sizeof(WCHAR))))
+        if (!(*description = LocalAlloc( 0, info->Description.Length + sizeof(WCHAR))))
             status = STATUS_NO_MEMORY;
         else
         {
-            if (length)
-                memcpy(*description, info->Description, length);
-            (*description)[length / sizeof(WCHAR)] = 0;
+            if (info->Description.Length)
+                memcpy(*description, info->Description.Buffer, info->Description.Length);
+            (*description)[info->Description.Length / sizeof(WCHAR)] = 0;
         }
     }
 
@@ -868,7 +877,7 @@ static void CDECL start_fiber(void)
     __TRY
     {
         start( fiber->param );
-        ExitThread( 1 );
+        RtlExitUserThread( 1 );
     }
     __EXCEPT(UnhandledExceptionFilter)
     {
@@ -998,7 +1007,7 @@ void WINAPI DECLSPEC_HOTPATCH DeleteFiber( LPVOID fiber_ptr )
     if (fiber == NtCurrentTeb()->Tib.u.FiberData)
     {
         HeapFree( GetProcessHeap(), 0, fiber );
-        ExitThread(1);
+        RtlExitUserThread( 1 );
     }
     RtlFreeUserStack( fiber->stack_allocation );
     HeapFree( GetProcessHeap(), 0, fiber->fls_slots );
