@@ -33,8 +33,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
-static const WCHAR wszOutputPinName[] = { 'O','u','t','p','u','t',0 };
-
 static const AM_MEDIA_TYPE default_mt =
 {
     {0xe436eb83,0x524f,0x11ce,{0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70}},   /* MEDIATYPE_Stream */
@@ -48,12 +46,12 @@ static const AM_MEDIA_TYPE default_mt =
     NULL
 };
 
-typedef struct DATAREQUEST
+struct request
 {
-    IMediaSample *pSample;
-    DWORD_PTR dwUserData;
+    IMediaSample *sample;
+    DWORD_PTR cookie;
     OVERLAPPED ovl;
-} DATAREQUEST;
+};
 
 typedef struct AsyncReader
 {
@@ -64,20 +62,16 @@ typedef struct AsyncReader
     IAsyncReader IAsyncReader_iface;
 
     LPOLESTR pszFileName;
-    AM_MEDIA_TYPE *pmt;
+    AM_MEDIA_TYPE mt;
     ALLOCATOR_PROPERTIES allocProps;
-    HANDLE file;
-    BOOL flushing;
-    unsigned int queued_number;
-    unsigned int samples;
-    unsigned int oldest_sample;
+    HANDLE file, port, io_thread;
     CRITICAL_SECTION sample_cs;
-    DATAREQUEST *sample_list;
-    /* Have a handle for every sample, and then one more as flushing handle */
-    HANDLE *handle_list;
+    BOOL flushing;
+    struct request *requests;
+    unsigned int max_requests;
+    CONDITION_VARIABLE sample_cv;
 } AsyncReader;
 
-static const IPinVtbl FileAsyncReaderPin_Vtbl;
 static const struct strmbase_source_ops source_ops;
 
 static inline AsyncReader *impl_from_strmbase_filter(struct strmbase_filter *iface)
@@ -90,17 +84,9 @@ static inline AsyncReader *impl_from_IFileSourceFilter(IFileSourceFilter *iface)
     return CONTAINING_RECORD(iface, AsyncReader, IFileSourceFilter_iface);
 }
 
-static const IBaseFilterVtbl AsyncReader_Vtbl;
 static const IFileSourceFilterVtbl FileSource_Vtbl;
 static const IAsyncReaderVtbl FileAsyncReader_Vtbl;
 
-static const WCHAR mediatype_name[] = {
-    'M', 'e', 'd', 'i', 'a', ' ', 'T', 'y', 'p', 'e', 0 };
-static const WCHAR subtype_name[] = {
-    'S', 'u', 'b', 't', 'y', 'p', 'e', 0 };
-static const WCHAR source_filter_name[] = {
-    'S','o','u','r','c','e',' ','F','i','l','t','e','r',0};
-
 static unsigned char byte_from_hex_char(WCHAR wHex)
 {
     switch (towlower(wHex))
@@ -232,15 +218,15 @@ BOOL get_media_type(const WCHAR *filename, GUID *majortype, GUID *subtype, GUID
         if (!RegOpenKeyExW(HKEY_CLASSES_ROOT, extensions_path, 0, KEY_READ, &key))
         {
             size = sizeof(guidstr);
-            if (majortype && !RegQueryValueExW(key, mediatype_name, NULL, NULL, (BYTE *)guidstr, &size))
+            if (majortype && !RegQueryValueExW(key, L"Media Type", NULL, NULL, (BYTE *)guidstr, &size))
                 CLSIDFromString(guidstr, majortype);
 
             size = sizeof(guidstr);
-            if (subtype && !RegQueryValueExW(key, subtype_name, NULL, NULL, (BYTE *)guidstr, &size))
+            if (subtype && !RegQueryValueExW(key, L"Subtype", NULL, NULL, (BYTE *)guidstr, &size))
                 CLSIDFromString(guidstr, subtype);
 
             size = sizeof(guidstr);
-            if (source_clsid && !RegQueryValueExW(key, source_filter_name, NULL, NULL, (BYTE *)guidstr, &size))
+            if (source_clsid && !RegQueryValueExW(key, L"Source Filter", NULL, NULL, (BYTE *)guidstr, &size))
                 CLSIDFromString(guidstr, source_clsid);
 
             RegCloseKey(key);
@@ -306,7 +292,7 @@ BOOL get_media_type(const WCHAR *filename, GUID *majortype, GUID *subtype, GUID
                         NULL, NULL, (BYTE *)pattern, &max_size))
                     break;
 
-                if (!wcscmp(value_name, source_filter_name))
+                if (!wcscmp(value_name, L"Source Filter"))
                     continue;
 
                 if (!process_pattern_string(pattern, file))
@@ -317,7 +303,7 @@ BOOL get_media_type(const WCHAR *filename, GUID *majortype, GUID *subtype, GUID
                 if (subtype)
                     CLSIDFromString(subtype_str, subtype);
                 size = sizeof(source_clsid_str);
-                if (source_clsid && !RegQueryValueExW(subtype_key, source_filter_name,
+                if (source_clsid && !RegQueryValueExW(subtype_key, L"Source Filter",
                         NULL, NULL, (BYTE *)source_clsid_str, &size))
                     CLSIDFromString(source_clsid_str, source_clsid);
 
@@ -363,21 +349,26 @@ static void async_reader_destroy(struct strmbase_filter *iface)
 
         IPin_Disconnect(&filter->source.pin.IPin_iface);
 
-        CoTaskMemFree(filter->sample_list);
-        if (filter->handle_list)
+        if (filter->requests)
         {
-            for (i = 0; i <= filter->samples; ++i)
-                CloseHandle(filter->handle_list[i]);
-            CoTaskMemFree(filter->handle_list);
+            for (i = 0; i < filter->max_requests; ++i)
+                CloseHandle(filter->requests[i].ovl.hEvent);
+            free(filter->requests);
         }
         CloseHandle(filter->file);
         filter->sample_cs.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&filter->sample_cs);
         strmbase_source_cleanup(&filter->source);
+
+        CoTaskMemFree(filter->pszFileName);
+        FreeMediaType(&filter->mt);
     }
-    CoTaskMemFree(filter->pszFileName);
-    if (filter->pmt)
-        DeleteMediaType(filter->pmt);
+
+    PostQueuedCompletionStatus(filter->port, 0, 1, NULL);
+    WaitForSingleObject(filter->io_thread, INFINITE);
+    CloseHandle(filter->io_thread);
+    CloseHandle(filter->port);
+
     strmbase_filter_cleanup(&filter->filter);
     CoTaskMemFree(filter);
 }
@@ -403,6 +394,42 @@ static const struct strmbase_filter_ops filter_ops =
     .filter_query_interface = async_reader_query_interface,
 };
 
+static DWORD CALLBACK io_thread(void *arg)
+{
+    AsyncReader *filter = arg;
+    struct request *req;
+    OVERLAPPED *ovl;
+    ULONG_PTR key;
+    DWORD size;
+    BOOL ret;
+
+    for (;;)
+    {
+        ret = GetQueuedCompletionStatus(filter->port, &size, &key, &ovl, INFINITE);
+
+        if (ret && key)
+            break;
+
+        EnterCriticalSection(&filter->sample_cs);
+
+        req = CONTAINING_RECORD(ovl, struct request, ovl);
+        TRACE("Got sample %u.\n", req - filter->requests);
+        assert(req >= filter->requests && req < filter->requests + filter->max_requests);
+
+        if (ret)
+            WakeConditionVariable(&filter->sample_cv);
+        else
+        {
+            ERR("GetQueuedCompletionStatus() returned failure, error %u.\n", GetLastError());
+            req->sample = NULL;
+        }
+
+        LeaveCriticalSection(&filter->sample_cs);
+    }
+
+    return 0;
+}
+
 HRESULT AsyncReader_create(IUnknown *outer, void **out)
 {
     AsyncReader *pAsyncRead;
@@ -412,14 +439,19 @@ HRESULT AsyncReader_create(IUnknown *outer, void **out)
     if (!pAsyncRead)
         return E_OUTOFMEMORY;
 
-    strmbase_filter_init(&pAsyncRead->filter, &AsyncReader_Vtbl, outer, &CLSID_AsyncReader, &filter_ops);
+    strmbase_filter_init(&pAsyncRead->filter, outer, &CLSID_AsyncReader, &filter_ops);
 
     pAsyncRead->IFileSourceFilter_iface.lpVtbl = &FileSource_Vtbl;
 
     pAsyncRead->IAsyncReader_iface.lpVtbl = &FileAsyncReader_Vtbl;
 
     pAsyncRead->pszFileName = NULL;
-    pAsyncRead->pmt = NULL;
+
+    InitializeCriticalSection(&pAsyncRead->sample_cs);
+    pAsyncRead->sample_cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": FileAsyncReader.sample_cs");
+    InitializeConditionVariable(&pAsyncRead->sample_cv);
+    pAsyncRead->port = CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL, 0, 0);
+    pAsyncRead->io_thread = CreateThread(NULL, 0, io_thread, pAsyncRead, 0, NULL);
 
     *out = &pAsyncRead->filter.IUnknown_inner;
 
@@ -428,25 +460,6 @@ HRESULT AsyncReader_create(IUnknown *outer, void **out)
     return S_OK;
 }
 
-static const IBaseFilterVtbl AsyncReader_Vtbl =
-{
-    BaseFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    BaseFilterImpl_Stop,
-    BaseFilterImpl_Pause,
-    BaseFilterImpl_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
-};
-
 static HRESULT WINAPI FileSource_QueryInterface(IFileSourceFilter * iface, REFIID riid, LPVOID * ppv)
 {
     AsyncReader *This = impl_from_IFileSourceFilter(iface);
@@ -488,46 +501,45 @@ static HRESULT WINAPI FileSource_Load(IFileSourceFilter * iface, LPCOLESTR pszFi
         return HRESULT_FROM_WIN32(GetLastError());
     }
 
-    strmbase_source_init(&This->source, &FileAsyncReaderPin_Vtbl, &This->filter,
-            wszOutputPinName, &source_ops);
+    if (This->pszFileName)
+    {
+        free(This->pszFileName);
+        FreeMediaType(&This->mt);
+    }
+
+    if (!(This->pszFileName = wcsdup(pszFileName)))
+    {
+        CloseHandle(hFile);
+        return E_OUTOFMEMORY;
+    }
+
+    strmbase_source_init(&This->source, &This->filter, L"Output", &source_ops);
     BaseFilterImpl_IncrementPinVersion(&This->filter);
 
     This->file = hFile;
     This->flushing = FALSE;
-    This->sample_list = NULL;
-    This->handle_list = NULL;
-    This->queued_number = 0;
-    InitializeCriticalSection(&This->sample_cs);
-    This->sample_cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": FileAsyncReader.sample_cs");
+    This->requests = NULL;
 
-    CoTaskMemFree(This->pszFileName);
-    if (This->pmt)
-        DeleteMediaType(This->pmt);
-
-    This->pszFileName = CoTaskMemAlloc((lstrlenW(pszFileName) + 1) * sizeof(WCHAR));
-    lstrcpyW(This->pszFileName, pszFileName);
-
-    This->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
     if (!pmt)
     {
-        CopyMediaType(This->pmt, &default_mt);
-        if (get_media_type(pszFileName, &This->pmt->majortype, &This->pmt->subtype, NULL))
+        CopyMediaType(&This->mt, &default_mt);
+        if (get_media_type(pszFileName, &This->mt.majortype, &This->mt.subtype, NULL))
         {
             TRACE("Found major type %s, subtype %s.\n",
-                    debugstr_guid(&This->pmt->majortype), debugstr_guid(&This->pmt->subtype));
+                    debugstr_guid(&This->mt.majortype), debugstr_guid(&This->mt.subtype));
         }
     }
     else
-        CopyMediaType(This->pmt, pmt);
+        CopyMediaType(&This->mt, pmt);
 
     return S_OK;
 }
 
-static HRESULT WINAPI FileSource_GetCurFile(IFileSourceFilter * iface, LPOLESTR * ppszFileName, AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI FileSource_GetCurFile(IFileSourceFilter *iface, LPOLESTR *ppszFileName, AM_MEDIA_TYPE *mt)
 {
     AsyncReader *This = impl_from_IFileSourceFilter(iface);
-    
-    TRACE("%p->(%p, %p)\n", This, ppszFileName, pmt);
+
+    TRACE("filter %p, filename %p, mt %p.\n", This, ppszFileName, mt);
 
     if (!ppszFileName)
         return E_POINTER;
@@ -535,18 +547,16 @@ static HRESULT WINAPI FileSource_GetCurFile(IFileSourceFilter * iface, LPOLESTR
     /* copy file name & media type if available, otherwise clear the outputs */
     if (This->pszFileName)
     {
-        *ppszFileName = CoTaskMemAlloc((lstrlenW(This->pszFileName) + 1) * sizeof(WCHAR));
-        lstrcpyW(*ppszFileName, This->pszFileName);
+        *ppszFileName = CoTaskMemAlloc((wcslen(This->pszFileName) + 1) * sizeof(WCHAR));
+        wcscpy(*ppszFileName, This->pszFileName);
+        if (mt)
+            CopyMediaType(mt, &This->mt);
     }
     else
-        *ppszFileName = NULL;
-
-    if (pmt)
     {
-        if (This->pmt)
-            CopyMediaType(pmt, This->pmt);
-        else
-            ZeroMemory(pmt, sizeof(*pmt));
+        *ppszFileName = NULL;
+        if (mt)
+            memset(mt, 0, sizeof(AM_MEDIA_TYPE));
     }
 
     return S_OK;
@@ -576,12 +586,12 @@ static inline AsyncReader *impl_from_IAsyncReader(IAsyncReader *iface)
     return CONTAINING_RECORD(iface, AsyncReader, IAsyncReader_iface);
 }
 
-static HRESULT source_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *pmt)
+static HRESULT source_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *mt)
 {
     AsyncReader *filter = impl_from_strmbase_pin(iface);
 
-    if (IsEqualGUID(&pmt->majortype, &filter->pmt->majortype) &&
-        IsEqualGUID(&pmt->subtype, &filter->pmt->subtype))
+    if (IsEqualGUID(&mt->majortype, &filter->mt.majortype)
+            && IsEqualGUID(&mt->subtype, &filter->mt.subtype))
         return S_OK;
 
     return S_FALSE;
@@ -595,7 +605,7 @@ static HRESULT source_get_media_type(struct strmbase_pin *iface, unsigned int in
         return VFW_S_NO_MORE_ITEMS;
 
     if (index == 0)
-        CopyMediaType(mt, filter->pmt);
+        CopyMediaType(mt, &filter->mt);
     else if (index == 1)
         CopyMediaType(mt, &default_mt);
     return S_OK;
@@ -614,28 +624,6 @@ static HRESULT source_query_interface(struct strmbase_pin *iface, REFIID iid, vo
     return S_OK;
 }
 
-static const IPinVtbl FileAsyncReaderPin_Vtbl = 
-{
-    BasePinImpl_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseOutputPinImpl_Connect,
-    BaseOutputPinImpl_ReceiveConnection,
-    BasePinImpl_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    BaseOutputPinImpl_EndOfStream,
-    BaseOutputPinImpl_BeginFlush,
-    BaseOutputPinImpl_EndFlush,
-    BasePinImpl_NewSegment
-};
-
 /* Function called as a helper to IPin_Connect */
 /* specific AM_MEDIA_TYPE - it cannot be NULL */
 /* this differs from standard OutputPin_AttemptConnection only in that it
@@ -712,308 +700,157 @@ static ULONG WINAPI FileAsyncReader_Release(IAsyncReader * iface)
     return IPin_Release(&filter->source.pin.IPin_iface);
 }
 
-#define DEF_ALIGNMENT 1
-
-static HRESULT WINAPI FileAsyncReader_RequestAllocator(IAsyncReader * iface, IMemAllocator * pPreferred, ALLOCATOR_PROPERTIES * pProps, IMemAllocator ** ppActual)
+static HRESULT WINAPI FileAsyncReader_RequestAllocator(IAsyncReader *iface,
+        IMemAllocator *preferred, ALLOCATOR_PROPERTIES *props, IMemAllocator **ret_allocator)
 {
-    AsyncReader *This = impl_from_IAsyncReader(iface);
-    HRESULT hr = S_OK;
+    AsyncReader *filter = impl_from_IAsyncReader(iface);
+    IMemAllocator *allocator;
+    unsigned int i;
+    HRESULT hr;
 
-    TRACE("%p->(%p, %p, %p)\n", This, pPreferred, pProps, ppActual);
+    TRACE("filter %p, preferred %p, props %p, ret_allocator %p.\n", filter, preferred, props, ret_allocator);
 
-    if (!pProps->cbAlign || (pProps->cbAlign % DEF_ALIGNMENT) != 0)
-        pProps->cbAlign = DEF_ALIGNMENT;
+    if (!props->cbAlign)
+        props->cbAlign = 1;
 
-    if (pPreferred)
+    *ret_allocator = NULL;
+
+    if (preferred)
+        IMemAllocator_AddRef(allocator = preferred);
+    else if (FAILED(hr = CoCreateInstance(&CLSID_MemoryAllocator, NULL,
+            CLSCTX_INPROC, &IID_IMemAllocator, (void **)&allocator)))
+        return hr;
+
+    if (FAILED(hr = IMemAllocator_SetProperties(allocator, props, props)))
     {
-        hr = IMemAllocator_SetProperties(pPreferred, pProps, pProps);
-        /* FIXME: check we are still aligned */
-        if (SUCCEEDED(hr))
-        {
-            IMemAllocator_AddRef(pPreferred);
-            *ppActual = pPreferred;
-            TRACE("FileAsyncReader_RequestAllocator -- %x\n", hr);
-            goto done;
-        }
+        IMemAllocator_Release(allocator);
+        return hr;
     }
 
-    pPreferred = NULL;
-
-    hr = CoCreateInstance(&CLSID_MemoryAllocator, NULL, CLSCTX_INPROC, &IID_IMemAllocator, (LPVOID *)&pPreferred);
-
-    if (SUCCEEDED(hr))
+    if (filter->requests)
     {
-        hr = IMemAllocator_SetProperties(pPreferred, pProps, pProps);
-        /* FIXME: check we are still aligned */
-        if (SUCCEEDED(hr))
-        {
-            *ppActual = pPreferred;
-            TRACE("FileAsyncReader_RequestAllocator -- %x\n", hr);
-        }
+        for (i = 0; i < filter->max_requests; ++i)
+            CloseHandle(filter->requests[i].ovl.hEvent);
+        free(filter->requests);
     }
 
-done:
-    if (SUCCEEDED(hr))
+    filter->max_requests = props->cBuffers;
+    TRACE("Maximum request count: %u.\n", filter->max_requests);
+    if (!(filter->requests = calloc(filter->max_requests, sizeof(filter->requests[0]))))
     {
-        CoTaskMemFree(This->sample_list);
-        if (This->handle_list)
-        {
-            int x;
-            for (x = 0; x <= This->samples; ++x)
-                CloseHandle(This->handle_list[x]);
-            CoTaskMemFree(This->handle_list);
-        }
-
-        This->samples = pProps->cBuffers;
-        This->oldest_sample = 0;
-        TRACE("Samples: %u\n", This->samples);
-        This->sample_list = CoTaskMemAlloc(sizeof(This->sample_list[0]) * pProps->cBuffers);
-        This->handle_list = CoTaskMemAlloc(sizeof(HANDLE) * pProps->cBuffers * 2);
-
-        if (This->sample_list && This->handle_list)
-        {
-            int x;
-            ZeroMemory(This->sample_list, sizeof(This->sample_list[0]) * pProps->cBuffers);
-            for (x = 0; x < This->samples; ++x)
-            {
-                This->sample_list[x].ovl.hEvent = This->handle_list[x] = CreateEventW(NULL, 0, 0, NULL);
-                if (x + 1 < This->samples)
-                    This->handle_list[This->samples + 1 + x] = This->handle_list[x];
-            }
-            This->handle_list[This->samples] = CreateEventW(NULL, 1, 0, NULL);
-            This->allocProps = *pProps;
-        }
-        else
-        {
-            hr = E_OUTOFMEMORY;
-            CoTaskMemFree(This->sample_list);
-            CoTaskMemFree(This->handle_list);
-            This->samples = 0;
-            This->sample_list = NULL;
-            This->handle_list = NULL;
-        }
+        IMemAllocator_Release(allocator);
+        return E_OUTOFMEMORY;
     }
 
-    if (FAILED(hr))
-    {
-        *ppActual = NULL;
-        if (pPreferred)
-            IMemAllocator_Release(pPreferred);
-    }
+    for (i = 0; i < filter->max_requests; ++i)
+        filter->requests[i].ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
+    filter->allocProps = *props;
 
-    TRACE("-- %x\n", hr);
-    return hr;
+    *ret_allocator = allocator;
+    return S_OK;
 }
 
-/* we could improve the Request/WaitForNext mechanism by allowing out of order samples.
- * however, this would be quite complicated to do and may be a bit error prone */
-static HRESULT WINAPI FileAsyncReader_Request(IAsyncReader * iface, IMediaSample * pSample, DWORD_PTR dwUser)
+static HRESULT WINAPI FileAsyncReader_Request(IAsyncReader *iface, IMediaSample *sample, DWORD_PTR cookie)
 {
-    AsyncReader *This = impl_from_IAsyncReader(iface);
-    HRESULT hr = S_OK;
-    REFERENCE_TIME Start;
-    REFERENCE_TIME Stop;
-    LPBYTE pBuffer = NULL;
+    AsyncReader *filter = impl_from_IAsyncReader(iface);
+    REFERENCE_TIME start, end;
+    struct request *req;
+    unsigned int i;
+    HRESULT hr;
+    BYTE *data;
 
-    TRACE("%p->(%p, %lx)\n", This, pSample, dwUser);
+    TRACE("filter %p, sample %p, cookie %#lx.\n", filter, sample, cookie);
 
-    if (!pSample)
+    if (!sample)
         return E_POINTER;
 
-    /* get start and stop positions in bytes */
-    if (SUCCEEDED(hr))
-        hr = IMediaSample_GetTime(pSample, &Start, &Stop);
+    if (FAILED(hr = IMediaSample_GetTime(sample, &start, &end)))
+        return hr;
 
-    if (SUCCEEDED(hr))
-        hr = IMediaSample_GetPointer(pSample, &pBuffer);
+    if (FAILED(hr = IMediaSample_GetPointer(sample, &data)))
+        return hr;
 
-    EnterCriticalSection(&This->sample_cs);
-    if (This->flushing)
+    EnterCriticalSection(&filter->sample_cs);
+    if (filter->flushing)
     {
-        LeaveCriticalSection(&This->sample_cs);
+        LeaveCriticalSection(&filter->sample_cs);
         return VFW_E_WRONG_STATE;
     }
 
-    if (SUCCEEDED(hr))
+    for (i = 0; i < filter->max_requests; ++i)
     {
-        DWORD dwLength = (DWORD) BYTES_FROM_MEDIATIME(Stop - Start);
-        DATAREQUEST *pDataRq;
-        int x;
-
-        /* Try to insert above the waiting sample if possible */
-        for (x = This->oldest_sample; x < This->samples; ++x)
-        {
-            if (!This->sample_list[x].pSample)
-                break;
-        }
-
-        if (x >= This->samples)
-            for (x = 0; x < This->oldest_sample; ++x)
-            {
-                if (!This->sample_list[x].pSample)
-                    break;
-            }
-
-        /* There must be a sample we have found */
-        assert(x < This->samples);
-        ++This->queued_number;
-
-        pDataRq = This->sample_list + x;
-
-        pDataRq->ovl.u.s.Offset = (DWORD) BYTES_FROM_MEDIATIME(Start);
-        pDataRq->ovl.u.s.OffsetHigh = (DWORD)(BYTES_FROM_MEDIATIME(Start) >> (sizeof(DWORD) * 8));
-        pDataRq->dwUserData = dwUser;
-
-        /* we violate traditional COM rules here by maintaining
-         * a reference to the sample, but not calling AddRef, but
-         * that's what MSDN says to do */
-        pDataRq->pSample = pSample;
-
-        /* this is definitely not how it is implemented on Win9x
-         * as they do not support async reads on files, but it is
-         * sooo much easier to use this than messing around with threads!
-         */
-        if (!ReadFile(This->file, pBuffer, dwLength, NULL, &pDataRq->ovl))
-            hr = HRESULT_FROM_WIN32(GetLastError());
-
-        /* ERROR_IO_PENDING is not actually an error since this is what we want! */
-        if (hr == HRESULT_FROM_WIN32(ERROR_IO_PENDING))
-            hr = S_OK;
+        if (!filter->requests[i].sample)
+            break;
     }
+    assert(i < filter->max_requests);
+    req = &filter->requests[i];
 
-    LeaveCriticalSection(&This->sample_cs);
+    req->ovl.u.s.Offset = BYTES_FROM_MEDIATIME(start);
+    req->ovl.u.s.OffsetHigh = BYTES_FROM_MEDIATIME(start) >> 32;
+    /* No reference is taken. */
 
-    TRACE("-- %x\n", hr);
+    if (ReadFile(filter->file, data, BYTES_FROM_MEDIATIME(end - start), NULL, &req->ovl)
+            || GetLastError() == ERROR_IO_PENDING)
+    {
+        hr = S_OK;
+        req->sample = sample;
+        req->cookie = cookie;
+    }
+    else
+        hr = HRESULT_FROM_WIN32(GetLastError());
+
+    LeaveCriticalSection(&filter->sample_cs);
     return hr;
 }
 
-static HRESULT WINAPI FileAsyncReader_WaitForNext(IAsyncReader * iface, DWORD dwTimeout, IMediaSample ** ppSample, DWORD_PTR * pdwUser)
+static HRESULT WINAPI FileAsyncReader_WaitForNext(IAsyncReader *iface,
+        DWORD timeout, IMediaSample **sample, DWORD_PTR *cookie)
 {
-    AsyncReader *This = impl_from_IAsyncReader(iface);
-    HRESULT hr = S_OK;
-    DWORD buffer = ~0;
+    AsyncReader *filter = impl_from_IAsyncReader(iface);
+    unsigned int i;
 
-    TRACE("%p->(%u, %p, %p)\n", This, dwTimeout, ppSample, pdwUser);
+    TRACE("filter %p, timeout %u, sample %p, cookie %p.\n", filter, timeout, sample, cookie);
 
-    *ppSample = NULL;
-    *pdwUser = 0;
+    *sample = NULL;
+    *cookie = 0;
 
-    EnterCriticalSection(&This->sample_cs);
-    if (!This->flushing)
+    EnterCriticalSection(&filter->sample_cs);
+
+    do
     {
-        LONG oldest = This->oldest_sample;
-
-        if (!This->queued_number)
+        if (filter->flushing)
         {
-            /* It could be that nothing is queued right now, but that can be fixed */
-            WARN("Called without samples in queue and not flushing!!\n");
-        }
-        LeaveCriticalSection(&This->sample_cs);
-
-        /* wait for an object to read, or time out */
-        buffer = WaitForMultipleObjectsEx(This->samples+1, This->handle_list + oldest, FALSE, dwTimeout, TRUE);
-
-        EnterCriticalSection(&This->sample_cs);
-        if (buffer <= This->samples)
-        {
-            /* Re-scale the buffer back to normal */
-            buffer += oldest;
-
-            /* Uh oh, we overshot the flusher handle, renormalize it back to 0..Samples-1 */
-            if (buffer > This->samples)
-                buffer -= This->samples + 1;
-            assert(buffer <= This->samples);
+            LeaveCriticalSection(&filter->sample_cs);
+            return VFW_E_WRONG_STATE;
         }
 
-        if (buffer >= This->samples)
+        for (i = 0; i < filter->max_requests; ++i)
         {
-            if (buffer != This->samples)
+            struct request *req = &filter->requests[i];
+            DWORD size;
+
+            if (req->sample && GetOverlappedResult(filter->file, &req->ovl, &size, FALSE))
             {
-                FIXME("Returned: %u (%08x)\n", buffer, GetLastError());
-                hr = VFW_E_TIMEOUT;
-            }
-            else
-                hr = VFW_E_WRONG_STATE;
-            buffer = ~0;
-        }
-        else
-            --This->queued_number;
-    }
+                REFERENCE_TIME start, end;
 
-    if (This->flushing && buffer == ~0)
-    {
-        for (buffer = 0; buffer < This->samples; ++buffer)
-        {
-            if (This->sample_list[buffer].pSample)
-            {
-                ResetEvent(This->handle_list[buffer]);
-                break;
+                IMediaSample_SetActualDataLength(req->sample, size);
+                start = MEDIATIME_FROM_BYTES(((ULONGLONG)req->ovl.u.s.OffsetHigh << 32) + req->ovl.u.s.Offset);
+                end = start + MEDIATIME_FROM_BYTES(size);
+                IMediaSample_SetTime(req->sample, &start, &end);
+
+                *sample = req->sample;
+                *cookie = req->cookie;
+                req->sample = NULL;
+
+                LeaveCriticalSection(&filter->sample_cs);
+                TRACE("Returning sample %u.\n", i);
+                return S_OK;
             }
         }
-        if (buffer == This->samples)
-        {
-            assert(!This->queued_number);
-            hr = VFW_E_TIMEOUT;
-        }
-        else
-        {
-            --This->queued_number;
-            hr = S_OK;
-        }
-    }
+    } while (SleepConditionVariableCS(&filter->sample_cv, &filter->sample_cs, timeout));
 
-    if (SUCCEEDED(hr))
-    {
-        REFERENCE_TIME rtStart, rtStop;
-        DATAREQUEST *pDataRq = This->sample_list + buffer;
-        DWORD dwBytes = 0;
-
-        /* get any errors */
-        if (!This->flushing && !GetOverlappedResult(This->file, &pDataRq->ovl, &dwBytes, FALSE))
-            hr = HRESULT_FROM_WIN32(GetLastError());
-
-        /* Return the sample no matter what so it can be destroyed */
-        *ppSample = pDataRq->pSample;
-        *pdwUser = pDataRq->dwUserData;
-
-        if (This->flushing)
-            hr = VFW_E_WRONG_STATE;
-
-        if (FAILED(hr))
-            dwBytes = 0;
-
-        /* Set the time on the sample */
-        IMediaSample_SetActualDataLength(pDataRq->pSample, dwBytes);
-
-        rtStart = (DWORD64)pDataRq->ovl.u.s.Offset + ((DWORD64)pDataRq->ovl.u.s.OffsetHigh << 32);
-        rtStart = MEDIATIME_FROM_BYTES(rtStart);
-        rtStop = rtStart + MEDIATIME_FROM_BYTES(dwBytes);
-
-        IMediaSample_SetTime(pDataRq->pSample, &rtStart, &rtStop);
-
-        This->sample_list[buffer].pSample = NULL;
-        assert(This->oldest_sample < This->samples);
-
-        if (buffer == This->oldest_sample)
-        {
-            LONG x;
-            for (x = This->oldest_sample + 1; x < This->samples; ++x)
-                if (This->sample_list[x].pSample)
-                    break;
-            if (x >= This->samples)
-                for (x = 0; x < This->oldest_sample; ++x)
-                    if (This->sample_list[x].pSample)
-                        break;
-            if (This->oldest_sample == x)
-                /* No samples found, reset to 0 */
-                x = 0;
-            This->oldest_sample = x;
-        }
-    }
-    LeaveCriticalSection(&This->sample_cs);
-
-    TRACE("-- %x\n", hr);
-    return hr;
+    LeaveCriticalSection(&filter->sample_cs);
+    return VFW_E_TIMEOUT;
 }
 
 static BOOL sync_read(HANDLE file, LONGLONG offset, LONG length, BYTE *buffer, DWORD *read_len)
@@ -1021,7 +858,7 @@ static BOOL sync_read(HANDLE file, LONGLONG offset, LONG length, BYTE *buffer, D
     OVERLAPPED ovl = {0};
     BOOL ret;
 
-    ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
+    ovl.hEvent = (HANDLE)((ULONG_PTR)CreateEventW(NULL, TRUE, FALSE, NULL) | 1);
     ovl.u.s.Offset = (DWORD)offset;
     ovl.u.s.OffsetHigh = offset >> 32;
 
@@ -1112,14 +949,17 @@ static HRESULT WINAPI FileAsyncReader_Length(IAsyncReader *iface, LONGLONG *tota
 static HRESULT WINAPI FileAsyncReader_BeginFlush(IAsyncReader * iface)
 {
     AsyncReader *filter = impl_from_IAsyncReader(iface);
+    unsigned int i;
 
     TRACE("iface %p.\n", iface);
 
     EnterCriticalSection(&filter->sample_cs);
 
     filter->flushing = TRUE;
+    for (i = 0; i < filter->max_requests; ++i)
+        filter->requests[i].sample = NULL;
     CancelIoEx(filter->file, NULL);
-    SetEvent(filter->handle_list[filter->samples]);
+    WakeAllConditionVariable(&filter->sample_cv);
 
     LeaveCriticalSection(&filter->sample_cs);
 
@@ -1129,16 +969,12 @@ static HRESULT WINAPI FileAsyncReader_BeginFlush(IAsyncReader * iface)
 static HRESULT WINAPI FileAsyncReader_EndFlush(IAsyncReader * iface)
 {
     AsyncReader *filter = impl_from_IAsyncReader(iface);
-    int x;
 
     TRACE("iface %p.\n", iface);
 
     EnterCriticalSection(&filter->sample_cs);
 
-    ResetEvent(filter->handle_list[filter->samples]);
     filter->flushing = FALSE;
-    for (x = 0; x < filter->samples; ++x)
-        assert(!filter->sample_list[x].pSample);
 
     LeaveCriticalSection(&filter->sample_cs);
 
