@@ -23,10 +23,6 @@
 #include "dshow.h"
 #include "wine/test.h"
 
-static const WCHAR avifile[] = {'t','e','s','t','.','a','v','i',0};
-
-static const WCHAR source_id[] = {'O','u','t','p','u','t',0};
-
 static IBaseFilter *create_file_source(void)
 {
     IBaseFilter *filter = NULL;
@@ -45,7 +41,7 @@ static WCHAR *load_resource(const WCHAR *name)
     void *ptr;
 
     GetTempPathW(ARRAY_SIZE(pathW), pathW);
-    lstrcatW(pathW, name);
+    wcscat(pathW, name);
 
     file = CreateFileW(pathW, GENERIC_READ|GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
     ok(file != INVALID_HANDLE_VALUE, "Failed to create file %s, error %u.\n",
@@ -97,7 +93,7 @@ static void check_interface_(unsigned int line, void *iface_ptr, REFIID iid, BOO
 
 static void test_interfaces(void)
 {
-    const WCHAR *filename = load_resource(avifile);
+    const WCHAR *filename = load_resource(L"test.avi");
     IBaseFilter *filter = create_file_source();
     IPin *pin;
 
@@ -118,7 +114,7 @@ static void test_interfaces(void)
     check_interface(filter, &IID_IVideoWindow, FALSE);
 
     load_file(filter, filename);
-    IBaseFilter_FindPin(filter, source_id, &pin);
+    IBaseFilter_FindPin(filter, L"Output", &pin);
 
     check_interface(pin, &IID_IAsyncReader, TRUE);
     check_interface(pin, &IID_IPin, TRUE);
@@ -229,7 +225,6 @@ static void test_aggregation(void)
 
 static void test_file_source_filter(void)
 {
-    static const WCHAR prefix[] = {'w','i','n',0};
     static const struct
     {
         const char *label;
@@ -297,7 +292,7 @@ static void test_file_source_filter(void)
     int i;
 
     GetTempPathW(MAX_PATH, temp);
-    GetTempFileNameW(temp, prefix, 0, path);
+    GetTempFileNameW(temp, L"win", 0, path);
 
     for (i = 0; i < ARRAY_SIZE(tests); i++)
     {
@@ -335,7 +330,7 @@ static void test_file_source_filter(void)
         memset(&file_mt, 0x11, sizeof(file_mt));
         hr = IFileSourceFilter_GetCurFile(filesource, &olepath, &file_mt);
         ok(hr == S_OK, "Got hr %#x.\n", hr);
-        ok(!lstrcmpW(olepath, path), "Expected path %s, got %s.\n",
+        ok(!wcscmp(olepath, path), "Expected path %s, got %s.\n",
                 wine_dbgstr_w(path), wine_dbgstr_w(olepath));
         ok(IsEqualGUID(&file_mt.majortype, &MEDIATYPE_Stream), "Got major type %s.\n",
                 wine_dbgstr_guid(&file_mt.majortype));
@@ -354,7 +349,7 @@ static void test_file_source_filter(void)
         ok(!file_mt.pbFormat, "Got format %p.\n", file_mt.pbFormat);
         CoTaskMemFree(olepath);
 
-        hr = IBaseFilter_FindPin(filter, source_id, &pin);
+        hr = IBaseFilter_FindPin(filter, L"Output", &pin);
         ok(hr == S_OK, "Got hr %#x.\n", hr);
 
         hr = IPin_EnumMediaTypes(pin, &enum_mt);
@@ -423,7 +418,7 @@ static void test_file_source_filter(void)
     mt.pUnk = NULL;
     mt.cbFormat = 0;
     mt.pbFormat = NULL;
-    filename = load_resource(avifile);
+    filename = load_resource(L"test.avi");
     hr = IFileSourceFilter_Load(filesource, filename, &mt);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
 
@@ -432,7 +427,7 @@ static void test_file_source_filter(void)
     ok(!memcmp(&file_mt, &mt, sizeof(mt)), "Media types did not match.\n");
     CoTaskMemFree(olepath);
 
-    hr = IBaseFilter_FindPin(filter, source_id, &pin);
+    hr = IBaseFilter_FindPin(filter, L"Output", &pin);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
 
     hr = IPin_EnumMediaTypes(pin, &enum_mt);
@@ -460,6 +455,7 @@ static void test_file_source_filter(void)
 
     hr = IPin_QueryAccept(pin, pmt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    CoTaskMemFree(pmt);
 
     hr = IEnumMediaTypes_Next(enum_mt, 1, &pmt, NULL);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
@@ -495,7 +491,7 @@ static void test_file_source_filter(void)
 
 static void test_enum_pins(void)
 {
-    const WCHAR *filename = load_resource(avifile);
+    const WCHAR *filename = load_resource(L"test.avi");
     IBaseFilter *filter = create_file_source();
     IEnumPins *enum1, *enum2;
     IPin *pins[2];
@@ -530,7 +526,6 @@ static void test_enum_pins(void)
     load_file(filter, filename);
 
     hr = IEnumPins_Next(enum1, 1, pins, NULL);
-todo_wine
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
 
     hr = IEnumPins_Reset(enum1);
@@ -539,10 +534,8 @@ todo_wine
     hr = IEnumPins_Next(enum1, 1, pins, NULL);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ref = get_refcount(filter);
-todo_wine
     ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(pins[0]);
-todo_wine
     ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(enum1);
     ok(ref == 1, "Got unexpected refcount %d.\n", ref);
@@ -608,7 +601,7 @@ todo_wine
 
 static void test_find_pin(void)
 {
-    const WCHAR *filename = load_resource(avifile);
+    const WCHAR *filename = load_resource(L"test.avi");
     IBaseFilter *filter = create_file_source();
     IEnumPins *enumpins;
     IPin *pin, *pin2;
@@ -616,15 +609,14 @@ static void test_find_pin(void)
     ULONG ref;
     BOOL ret;
 
-    hr = IBaseFilter_FindPin(filter, source_id, &pin);
+    hr = IBaseFilter_FindPin(filter, L"Output", &pin);
     ok(hr == VFW_E_NOT_FOUND, "Got hr %#x.\n", hr);
 
     load_file(filter, filename);
 
-    hr = IBaseFilter_FindPin(filter, source_id, &pin);
+    hr = IBaseFilter_FindPin(filter, L"Output", &pin);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ref = get_refcount(filter);
-todo_wine
     ok(ref == 2, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(pin);
     ok(ref == 2, "Got unexpected refcount %d.\n", ref);
@@ -647,7 +639,7 @@ todo_wine
 
 static void test_pin_info(void)
 {
-    const WCHAR *filename = load_resource(avifile);
+    const WCHAR *filename = load_resource(L"test.avi");
     IBaseFilter *filter = create_file_source();
     PIN_DIRECTION dir;
     PIN_INFO info;
@@ -659,10 +651,9 @@ static void test_pin_info(void)
 
     load_file(filter, filename);
 
-    hr = IBaseFilter_FindPin(filter, source_id, &pin);
+    hr = IBaseFilter_FindPin(filter, L"Output", &pin);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ref = get_refcount(filter);
-todo_wine
     ok(ref == 2, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(pin);
     ok(ref == 2, "Got unexpected refcount %d.\n", ref);
@@ -671,12 +662,10 @@ todo_wine
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ok(info.pFilter == filter, "Expected filter %p, got %p.\n", filter, info.pFilter);
     ok(info.dir == PINDIR_OUTPUT, "Got direction %d.\n", info.dir);
-    ok(!lstrcmpW(info.achName, source_id), "Got name %s.\n", wine_dbgstr_w(info.achName));
+    ok(!wcscmp(info.achName, L"Output"), "Got name %s.\n", wine_dbgstr_w(info.achName));
     ref = get_refcount(filter);
-todo_wine
     ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     ref = get_refcount(pin);
-todo_wine
     ok(ref == 3, "Got unexpected refcount %d.\n", ref);
     IBaseFilter_Release(info.pFilter);
 
@@ -686,7 +675,7 @@ todo_wine
 
     hr = IPin_QueryId(pin, &id);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(!lstrcmpW(id, source_id), "Got id %s.\n", wine_dbgstr_w(id));
+    ok(!wcscmp(id, L"Output"), "Got id %s.\n", wine_dbgstr_w(id));
     CoTaskMemFree(id);
 
     IPin_Release(pin);
@@ -696,66 +685,59 @@ todo_wine
     ok(ret, "Failed to delete file, error %u.\n", GetLastError());
 }
 
-static void test_filter_state(void)
+static void test_unconnected_filter_state(void)
 {
     IBaseFilter *filter = create_file_source();
-    IMediaControl *control;
-    IFilterGraph2 *graph;
     FILTER_STATE state;
     HRESULT hr;
     ULONG ref;
 
-    CoCreateInstance(&CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
-            &IID_IFilterGraph2, (void **)&graph);
-    IFilterGraph2_AddFilter(graph, filter, NULL);
-    IFilterGraph2_QueryInterface(graph, &IID_IMediaControl, (void **)&control);
+    hr = IBaseFilter_GetState(filter, 0, &state);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(state == State_Stopped, "Got state %u.\n", state);
+
+    hr = IBaseFilter_Pause(filter);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 
     hr = IBaseFilter_GetState(filter, 0, &state);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Stopped, "Got state %d.\n", state);
+    ok(state == State_Paused, "Got state %u.\n", state);
 
-    hr = IMediaControl_Run(control);
-todo_wine
+    hr = IBaseFilter_Run(filter, 0);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
-    ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Running, "Got state %d.\n", state);
 
-    hr = IMediaControl_Stop(control);
+    hr = IBaseFilter_GetState(filter, 0, &state);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
-    ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Stopped, "Got state %d.\n", state);
+    ok(state == State_Running, "Got state %u.\n", state);
 
-    hr = IMediaControl_Pause(control);
-todo_wine
+    hr = IBaseFilter_Pause(filter);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
-    ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Paused, "Got state %d.\n", state);
 
-    hr = IMediaControl_Run(control);
-todo_wine
+    hr = IBaseFilter_GetState(filter, 0, &state);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
-    ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Running, "Got state %d.\n", state);
+    ok(state == State_Paused, "Got state %u.\n", state);
 
-    hr = IMediaControl_Pause(control);
-todo_wine
+    hr = IBaseFilter_Stop(filter);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
-    ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Paused, "Got state %d.\n", state);
 
-    hr = IMediaControl_Stop(control);
+    hr = IBaseFilter_GetState(filter, 0, &state);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    hr = IBaseFilter_GetState(filter, 1000, &state);
+    ok(state == State_Stopped, "Got state %u.\n", state);
+
+    hr = IBaseFilter_Run(filter, 0);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(state == State_Stopped, "Got state %d.\n", state);
+
+    hr = IBaseFilter_GetState(filter, 0, &state);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(state == State_Running, "Got state %u.\n", state);
+
+    hr = IBaseFilter_Stop(filter);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IBaseFilter_GetState(filter, 0, &state);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(state == State_Stopped, "Got state %u.\n", state);
 
-    IMediaControl_Release(control);
-    IFilterGraph2_Release(graph);
     ref = IBaseFilter_Release(filter);
     ok(!ref, "Got outstanding refcount %d.\n", ref);
 }
@@ -814,12 +796,28 @@ static void test_sync_read_aligned(IAsyncReader *reader, IMemAllocator *allocato
     IMediaSample_Release(sample);
 }
 
+struct request_thread_params
+{
+    IAsyncReader *reader;
+    IMediaSample *sample;
+};
+
+static DWORD CALLBACK request_thread(void *arg)
+{
+    struct request_thread_params *params = arg;
+    HRESULT hr = IAsyncReader_Request(params->reader, params->sample, 123);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    return 0;
+}
+
 static void test_request(IAsyncReader *reader, IMemAllocator *allocator)
 {
     IMediaSample *sample, *sample2, *ret_sample;
+    struct request_thread_params params;
     REFERENCE_TIME start_time, end_time;
     BYTE *data, *data2;
     DWORD_PTR cookie;
+    HANDLE thread;
     HRESULT hr;
     LONG len;
     int i;
@@ -903,16 +901,38 @@ static void test_request(IAsyncReader *reader, IMemAllocator *allocator)
     for (i = 0; i < 88; i++)
         ok(data2[i] == (512 + i) % 111, "Got wrong byte %02x at %u.\n", data2[i], i);
 
+    params.reader = reader;
+    params.sample = sample;
+    thread = CreateThread(NULL, 0, request_thread, &params, 0, NULL);
+    ok(!WaitForSingleObject(thread, 1000), "Wait timed out.\n");
+    CloseHandle(thread);
+
+    hr = IAsyncReader_WaitForNext(reader, 1000, &ret_sample, &cookie);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(ret_sample == sample, "Samples didn't match.\n");
+    ok(cookie == 123, "Got cookie %lu.\n", cookie);
+
     IMediaSample_Release(sample);
     IMediaSample_Release(sample2);
 }
 
+static DWORD CALLBACK wait_thread(void *arg)
+{
+    IAsyncReader *reader = arg;
+    IMediaSample *sample;
+    DWORD_PTR cookie;
+    HRESULT hr = IAsyncReader_WaitForNext(reader, 2000, &sample, &cookie);
+    ok(hr == VFW_E_WRONG_STATE, "Got hr %#x.\n", hr);
+    return 0;
+}
+
 static void test_flush(IAsyncReader *reader, IMemAllocator *allocator)
 {
     REFERENCE_TIME start_time, end_time;
     IMediaSample *sample, *ret_sample;
     BYTE buffer[20], *data;
     DWORD_PTR cookie;
+    HANDLE thread;
     HRESULT hr;
     int i;
 
@@ -964,12 +984,23 @@ static void test_flush(IAsyncReader *reader, IMemAllocator *allocator)
     for (i = 0; i < 512; i++)
         ok(data[i] == i % 111, "Got wrong byte %02x at %u.\n", data[i], i);
 
+    thread = CreateThread(NULL, 0, wait_thread, reader, 0, NULL);
+    ok(WaitForSingleObject(thread, 100) == WAIT_TIMEOUT, "Expected timeout.\n");
+
+    hr = IAsyncReader_BeginFlush(reader);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(!WaitForSingleObject(thread, 1000), "Wait timed out.\n");
+    CloseHandle(thread);
+
+    hr = IAsyncReader_EndFlush(reader);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
     IMediaSample_Release(sample);
 }
 
 static void test_async_reader(void)
 {
-    ALLOCATOR_PROPERTIES req_props = {2, 1024, 512, 0}, ret_props;
+    ALLOCATOR_PROPERTIES req_props = {100, 1024, 512, 0}, ret_props;
     IBaseFilter *filter = create_file_source();
     IFileSourceFilter *filesource;
     LONGLONG length, available;
@@ -986,7 +1017,7 @@ static void test_async_reader(void)
     int i;
 
     GetTempPathW(ARRAY_SIZE(filename), filename);
-    lstrcatW(filename, avifile);
+    wcscat(filename, L"test.avi");
     file = CreateFileW(filename, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
     ok(file != INVALID_HANDLE_VALUE, "Failed to create file, error %u.\n", GetLastError());
     for (i = 0; i < 600; i++)
@@ -998,7 +1029,7 @@ static void test_async_reader(void)
 
     IBaseFilter_QueryInterface(filter, &IID_IFileSourceFilter, (void **)&filesource);
     IFileSourceFilter_Load(filesource, filename, NULL);
-    IBaseFilter_FindPin(filter, source_id, &pin);
+    IBaseFilter_FindPin(filter, L"Output", &pin);
 
     hr = IPin_QueryInterface(pin, &IID_IAsyncReader, (void **)&reader);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
@@ -1039,7 +1070,7 @@ static void test_async_reader(void)
     ret_props = req_props;
     hr = IAsyncReader_RequestAllocator(reader, NULL, &ret_props, &allocator);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
-    ok(ret_props.cBuffers == 2, "Got %d buffers.\n", ret_props.cBuffers);
+    ok(ret_props.cBuffers == 100, "Got %d buffers.\n", ret_props.cBuffers);
     ok(ret_props.cbBuffer == 1024, "Got size %d.\n", ret_props.cbBuffer);
     ok(ret_props.cbAlign == 512, "Got alignment %d.\n", ret_props.cbAlign);
     ok(ret_props.cbPrefix == 0, "Got prefix %d.\n", ret_props.cbPrefix);
@@ -1060,6 +1091,112 @@ static void test_async_reader(void)
     ok(ret, "Failed to delete file, error %u.\n", GetLastError());
 }
 
+static void test_enum_media_types(void)
+{
+    const WCHAR *filename = load_resource(L"test.avi");
+    IBaseFilter *filter = create_file_source();
+    IEnumMediaTypes *enum1, *enum2;
+    AM_MEDIA_TYPE *mts[3];
+    ULONG ref, count;
+    HRESULT hr;
+    IPin *pin;
+    BOOL ret;
+
+    load_file(filter, filename);
+
+    IBaseFilter_FindPin(filter, L"Output", &pin);
+
+    hr = IPin_EnumMediaTypes(pin, &enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    CoTaskMemFree(mts[0]);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    CoTaskMemFree(mts[0]);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, &count);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(count == 1, "Got count %u.\n", count);
+    CoTaskMemFree(mts[0]);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, &count);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(count == 1, "Got count %u.\n", count);
+    CoTaskMemFree(mts[0]);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, &count);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(!count, "Got count %u.\n", count);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 2, mts, &count);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(count == 2, "Got count %u.\n", count);
+    CoTaskMemFree(mts[0]);
+    CoTaskMemFree(mts[1]);
+
+    hr = IEnumMediaTypes_Next(enum1, 2, mts, &count);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(!count, "Got count %u.\n", count);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 3, mts, &count);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(count == 2, "Got count %u.\n", count);
+    CoTaskMemFree(mts[0]);
+    CoTaskMemFree(mts[1]);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Clone(enum1, &enum2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Skip(enum1, 3);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Skip(enum1, 1);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Reset(enum1);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Skip(enum1, 2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Skip(enum1, 1);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum1, 1, mts, NULL);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+
+    hr = IEnumMediaTypes_Next(enum2, 1, mts, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    CoTaskMemFree(mts[0]);
+
+    IEnumMediaTypes_Release(enum1);
+    IEnumMediaTypes_Release(enum2);
+    IPin_Release(pin);
+
+    ref = IBaseFilter_Release(filter);
+    ok(!ref, "Got outstanding refcount %d.\n", ref);
+    ret = DeleteFileW(filename);
+    ok(ret, "Failed to delete file, error %u.\n", GetLastError());
+}
+
 START_TEST(filesource)
 {
     CoInitialize(NULL);
@@ -1069,9 +1206,10 @@ START_TEST(filesource)
     test_enum_pins();
     test_find_pin();
     test_pin_info();
-    test_filter_state();
+    test_unconnected_filter_state();
     test_file_source_filter();
     test_async_reader();
+    test_enum_media_types();
 
     CoUninitialize();
 }
