@@ -27,6 +27,182 @@ static const IMemInputPinVtbl MemInputPin_Vtbl;
 
 typedef HRESULT (*SendPinFunc)( IPin *to, LPVOID arg );
 
+struct enum_media_types
+{
+    IEnumMediaTypes IEnumMediaTypes_iface;
+    LONG refcount;
+
+    unsigned int index, count;
+    struct strmbase_pin *pin;
+};
+
+static const IEnumMediaTypesVtbl enum_media_types_vtbl;
+
+static HRESULT enum_media_types_create(struct strmbase_pin *pin, IEnumMediaTypes **out)
+{
+    struct enum_media_types *object;
+    AM_MEDIA_TYPE mt;
+
+    if (!out)
+        return E_POINTER;
+
+    if (!(object = heap_alloc_zero(sizeof(*object))))
+    {
+        *out = NULL;
+        return E_OUTOFMEMORY;
+    }
+
+    object->IEnumMediaTypes_iface.lpVtbl = &enum_media_types_vtbl;
+    object->refcount = 1;
+    object->pin = pin;
+    IPin_AddRef(&pin->IPin_iface);
+
+    while (pin->pFuncsTable->pin_get_media_type(pin, object->count, &mt) == S_OK)
+    {
+        FreeMediaType(&mt);
+        ++object->count;
+    }
+
+    TRACE("Created enumerator %p.\n", object);
+    *out = &object->IEnumMediaTypes_iface;
+
+    return S_OK;
+}
+
+static struct enum_media_types *impl_from_IEnumMediaTypes(IEnumMediaTypes *iface)
+{
+    return CONTAINING_RECORD(iface, struct enum_media_types, IEnumMediaTypes_iface);
+}
+
+static HRESULT WINAPI enum_media_types_QueryInterface(IEnumMediaTypes *iface, REFIID iid, void **out)
+{
+    TRACE("iface %p, iid %s, out %p.\n", iface, debugstr_guid(iid), out);
+
+    if (IsEqualGUID(iid, &IID_IUnknown) || IsEqualGUID(iid, &IID_IEnumMediaTypes))
+    {
+        IEnumMediaTypes_AddRef(iface);
+        *out = iface;
+        return S_OK;
+    }
+
+    WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(iid));
+    *out = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI enum_media_types_AddRef(IEnumMediaTypes *iface)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+    ULONG refcount = InterlockedIncrement(&enummt->refcount);
+    TRACE("%p increasing refcount to %u.\n", enummt, refcount);
+    return refcount;
+}
+
+static ULONG WINAPI enum_media_types_Release(IEnumMediaTypes *iface)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+    ULONG refcount = InterlockedDecrement(&enummt->refcount);
+
+    TRACE("%p decreasing refcount to %u.\n", enummt, refcount);
+    if (!refcount)
+    {
+        IPin_Release(&enummt->pin->IPin_iface);
+        heap_free(enummt);
+    }
+    return refcount;
+}
+
+static HRESULT WINAPI enum_media_types_Next(IEnumMediaTypes *iface, ULONG count,
+        AM_MEDIA_TYPE **mts, ULONG *ret_count)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+    unsigned int i;
+    HRESULT hr;
+
+    TRACE("enummt %p, count %u, mts %p, ret_count %p.\n", enummt, count, mts, ret_count);
+
+    for (i = 0; i < count; ++i)
+    {
+        if ((mts[i] = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE))))
+            hr = enummt->pin->pFuncsTable->pin_get_media_type(enummt->pin, enummt->index + i, mts[i]);
+        else
+            hr = E_OUTOFMEMORY;
+        if (FAILED(hr))
+        {
+            while (i--)
+                DeleteMediaType(mts[i]);
+            *ret_count = 0;
+            return E_OUTOFMEMORY;
+        }
+        else if (hr != S_OK)
+            break;
+
+        if (TRACE_ON(strmbase))
+        {
+            TRACE("Returning media type %u:\n", enummt->index + i);
+            strmbase_dump_media_type(mts[i]);
+        }
+    }
+
+    if (count != 1 || ret_count)
+        *ret_count = i;
+    enummt->index += i;
+    return i == count ? S_OK : S_FALSE;
+}
+
+static HRESULT WINAPI enum_media_types_Skip(IEnumMediaTypes *iface, ULONG count)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+
+    TRACE("enummt %p, count %u.\n", enummt, count);
+
+    enummt->index += count;
+
+    return enummt->index > enummt->count ? S_FALSE : S_OK;
+}
+
+static HRESULT WINAPI enum_media_types_Reset(IEnumMediaTypes *iface)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+    AM_MEDIA_TYPE mt;
+
+    TRACE("enummt %p.\n", enummt);
+
+    enummt->count = 0;
+    while (enummt->pin->pFuncsTable->pin_get_media_type(enummt->pin, enummt->count, &mt) == S_OK)
+    {
+        FreeMediaType(&mt);
+        ++enummt->count;
+    }
+
+    enummt->index = 0;
+
+    return S_OK;
+}
+
+static HRESULT WINAPI enum_media_types_Clone(IEnumMediaTypes *iface, IEnumMediaTypes **out)
+{
+    struct enum_media_types *enummt = impl_from_IEnumMediaTypes(iface);
+    HRESULT hr;
+
+    TRACE("enummt %p, out %p.\n", enummt, out);
+
+    if (FAILED(hr = enum_media_types_create(enummt->pin, out)))
+        return hr;
+    return IEnumMediaTypes_Skip(*out, enummt->index);
+}
+
+static const IEnumMediaTypesVtbl enum_media_types_vtbl =
+{
+    enum_media_types_QueryInterface,
+    enum_media_types_AddRef,
+    enum_media_types_Release,
+    enum_media_types_Next,
+    enum_media_types_Skip,
+    enum_media_types_Reset,
+    enum_media_types_Clone,
+};
+
 static inline struct strmbase_pin *impl_from_IPin(IPin *iface)
 {
     return CONTAINING_RECORD(iface, struct strmbase_pin, IPin_iface);
@@ -82,7 +258,7 @@ HRESULT strmbase_pin_get_media_type(struct strmbase_pin *iface, unsigned int ind
     return VFW_S_NO_MORE_ITEMS;
 }
 
-HRESULT WINAPI BasePinImpl_QueryInterface(IPin *iface, REFIID iid, void **out)
+static HRESULT WINAPI pin_QueryInterface(IPin *iface, REFIID iid, void **out)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
     HRESULT hr;
@@ -107,49 +283,24 @@ HRESULT WINAPI BasePinImpl_QueryInterface(IPin *iface, REFIID iid, void **out)
     return S_OK;
 }
 
-ULONG WINAPI BasePinImpl_AddRef(IPin *iface)
+static ULONG WINAPI pin_AddRef(IPin *iface)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
     return IBaseFilter_AddRef(&pin->filter->IBaseFilter_iface);
 }
 
-ULONG WINAPI BasePinImpl_Release(IPin *iface)
+static ULONG WINAPI pin_Release(IPin *iface)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
     return IBaseFilter_Release(&pin->filter->IBaseFilter_iface);
 }
 
-HRESULT WINAPI BasePinImpl_Disconnect(IPin * iface)
+static HRESULT WINAPI pin_ConnectedTo(IPin * iface, IPin ** ppPin)
 {
     struct strmbase_pin *This = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p)->()\n", This);
-
-    EnterCriticalSection(&This->filter->csFilter);
-    {
-        if (This->peer)
-        {
-            IPin_Release(This->peer);
-            This->peer = NULL;
-            FreeMediaType(&This->mt);
-            ZeroMemory(&This->mt, sizeof(This->mt));
-            hr = S_OK;
-        }
-        else
-            hr = S_FALSE;
-    }
-    LeaveCriticalSection(&This->filter->csFilter);
-
-    return hr;
-}
-
-HRESULT WINAPI BasePinImpl_ConnectedTo(IPin * iface, IPin ** ppPin)
-{
-    struct strmbase_pin *This = impl_from_IPin(iface);
-    HRESULT hr;
-
-    TRACE("(%p)->(%p)\n", This, ppPin);
+    TRACE("pin %p %s:%s, peer %p.\n", This, debugstr_w(This->filter->name), debugstr_w(This->name), ppPin);
 
     EnterCriticalSection(&This->filter->csFilter);
     {
@@ -170,12 +321,12 @@ HRESULT WINAPI BasePinImpl_ConnectedTo(IPin * iface, IPin ** ppPin)
     return hr;
 }
 
-HRESULT WINAPI BasePinImpl_ConnectionMediaType(IPin * iface, AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI pin_ConnectionMediaType(IPin *iface, AM_MEDIA_TYPE *pmt)
 {
     struct strmbase_pin *This = impl_from_IPin(iface);
     HRESULT hr;
 
-    TRACE("(%p)->(%p)\n", This, pmt);
+    TRACE("pin %p %s:%s, pmt %p.\n", This, debugstr_w(This->filter->name), debugstr_w(This->name), pmt);
 
     EnterCriticalSection(&This->filter->csFilter);
     {
@@ -196,11 +347,11 @@ HRESULT WINAPI BasePinImpl_ConnectionMediaType(IPin * iface, AM_MEDIA_TYPE * pmt
     return hr;
 }
 
-HRESULT WINAPI BasePinImpl_QueryPinInfo(IPin *iface, PIN_INFO *info)
+static HRESULT WINAPI pin_QueryPinInfo(IPin *iface, PIN_INFO *info)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
 
-    TRACE("pin %p, info %p.\n", pin, info);
+    TRACE("pin %p %s:%s, info %p.\n", pin, debugstr_w(pin->filter->name), debugstr_w(pin->name), info);
 
     info->dir = pin->dir;
     IBaseFilter_AddRef(info->pFilter = &pin->filter->IBaseFilter_iface);
@@ -209,22 +360,22 @@ HRESULT WINAPI BasePinImpl_QueryPinInfo(IPin *iface, PIN_INFO *info)
     return S_OK;
 }
 
-HRESULT WINAPI BasePinImpl_QueryDirection(IPin *iface, PIN_DIRECTION *dir)
+static HRESULT WINAPI pin_QueryDirection(IPin *iface, PIN_DIRECTION *dir)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
 
-    TRACE("pin %p, dir %p.\n", pin, dir);
+    TRACE("pin %p %s:%s, dir %p.\n", pin, debugstr_w(pin->filter->name), debugstr_w(pin->name), dir);
 
     *dir = pin->dir;
 
     return S_OK;
 }
 
-HRESULT WINAPI BasePinImpl_QueryId(IPin *iface, WCHAR **id)
+static HRESULT WINAPI pin_QueryId(IPin *iface, WCHAR **id)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
 
-    TRACE("pin %p, id %p.\n", pin, id);
+    TRACE("pin %p %s:%s, id %p.\n", pin, debugstr_w(pin->filter->name), debugstr_w(pin->name), id);
 
     if (!(*id = CoTaskMemAlloc((lstrlenW(pin->name) + 1) * sizeof(WCHAR))))
         return E_OUTOFMEMORY;
@@ -234,23 +385,24 @@ HRESULT WINAPI BasePinImpl_QueryId(IPin *iface, WCHAR **id)
     return S_OK;
 }
 
-HRESULT WINAPI BasePinImpl_QueryAccept(IPin * iface, const AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI pin_QueryAccept(IPin *iface, const AM_MEDIA_TYPE *mt)
 {
-    struct strmbase_pin *This = impl_from_IPin(iface);
+    struct strmbase_pin *pin = impl_from_IPin(iface);
 
-    TRACE("(%p)->(%p)\n", iface, pmt);
-    strmbase_dump_media_type(pmt);
+    TRACE("pin %p %s:%s, mt %p.\n", pin, debugstr_w(pin->filter->name), debugstr_w(pin->name), mt);
+    strmbase_dump_media_type(mt);
 
-    return (This->pFuncsTable->pin_query_accept(This, pmt) == S_OK ? S_OK : S_FALSE);
+    return (pin->pFuncsTable->pin_query_accept(pin, mt) == S_OK ? S_OK : S_FALSE);
 }
 
-HRESULT WINAPI BasePinImpl_EnumMediaTypes(IPin *iface, IEnumMediaTypes **enum_media_types)
+static HRESULT WINAPI pin_EnumMediaTypes(IPin *iface, IEnumMediaTypes **enum_media_types)
 {
     struct strmbase_pin *pin = impl_from_IPin(iface);
     AM_MEDIA_TYPE mt;
     HRESULT hr;
 
-    TRACE("iface %p, enum_media_types %p.\n", iface, enum_media_types);
+    TRACE("pin %p %s:%s, enum_media_types %p.\n", pin, debugstr_w(pin->filter->name),
+            debugstr_w(pin->name), enum_media_types);
 
     if (FAILED(hr = pin->pFuncsTable->pin_get_media_type(pin, 0, &mt)))
         return hr;
@@ -260,23 +412,16 @@ HRESULT WINAPI BasePinImpl_EnumMediaTypes(IPin *iface, IEnumMediaTypes **enum_me
     return enum_media_types_create(pin, enum_media_types);
 }
 
-HRESULT WINAPI BasePinImpl_QueryInternalConnections(IPin * iface, IPin ** apPin, ULONG * cPin)
+static HRESULT WINAPI pin_QueryInternalConnections(IPin *iface, IPin **pins, ULONG *count)
 {
-    struct strmbase_pin *This = impl_from_IPin(iface);
+    struct strmbase_pin *pin = impl_from_IPin(iface);
 
-    TRACE("(%p)->(%p, %p)\n", This, apPin, cPin);
+    TRACE("pin %p %s:%s, pins %p, count %p.\n", pin, debugstr_w(pin->filter->name),
+            debugstr_w(pin->name), pins, count);
 
     return E_NOTIMPL; /* to tell caller that all input pins connected to all output pins */
 }
 
-HRESULT WINAPI BasePinImpl_NewSegment(IPin * iface, REFERENCE_TIME start, REFERENCE_TIME stop, double rate)
-{
-    TRACE("iface %p, start %s, stop %s, rate %.16e.\n",
-            iface, debugstr_time(start), debugstr_time(stop), rate);
-
-    return S_OK;
-}
-
 /*** OutputPin implementation ***/
 
 static inline struct strmbase_source *impl_source_from_IPin( IPin *iface )
@@ -284,12 +429,13 @@ static inline struct strmbase_source *impl_source_from_IPin( IPin *iface )
     return CONTAINING_RECORD(iface, struct strmbase_source, pin.IPin_iface);
 }
 
-HRESULT WINAPI BaseOutputPinImpl_Connect(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI source_Connect(IPin *iface, IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
 {
     HRESULT hr;
     struct strmbase_source *This = impl_source_from_IPin(iface);
 
-    TRACE("(%p)->(%p, %p)\n", This, pReceivePin, pmt);
+    TRACE("pin %p %s:%s, peer %p, mt %p.\n", This, debugstr_w(This->pin.filter->name),
+            debugstr_w(This->pin.name), pReceivePin, pmt);
     strmbase_dump_media_type(pmt);
 
     if (!pReceivePin)
@@ -375,18 +521,22 @@ HRESULT WINAPI BaseOutputPinImpl_Connect(IPin * iface, IPin * pReceivePin, const
     return hr;
 }
 
-HRESULT WINAPI BaseOutputPinImpl_ReceiveConnection(IPin *iface, IPin *pin, const AM_MEDIA_TYPE *pmt)
+static HRESULT WINAPI source_ReceiveConnection(IPin *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
 {
-    ERR("(%p)->(%p, %p) incoming connection on an output pin!\n", iface, pin, pmt);
+    struct strmbase_source *pin = impl_source_from_IPin(iface);
+
+    WARN("pin %p %s:%s, peer %p, mt %p, unexpected call.\n", pin,
+            debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name), peer, mt);
+
     return E_UNEXPECTED;
 }
 
-HRESULT WINAPI BaseOutputPinImpl_Disconnect(IPin * iface)
+static HRESULT WINAPI source_Disconnect(IPin *iface)
 {
     HRESULT hr;
     struct strmbase_source *This = impl_source_from_IPin(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("pin %p %s:%s.\n", This, debugstr_w(This->pin.filter->name), debugstr_w(This->pin.name));
 
     EnterCriticalSection(&This->pin.filter->csFilter);
     {
@@ -418,33 +568,71 @@ HRESULT WINAPI BaseOutputPinImpl_Disconnect(IPin * iface)
     return hr;
 }
 
-HRESULT WINAPI BaseOutputPinImpl_EndOfStream(IPin * iface)
+static HRESULT WINAPI source_EndOfStream(IPin *iface)
 {
-    TRACE("(%p)->()\n", iface);
+    struct strmbase_source *pin = impl_source_from_IPin(iface);
+
+    WARN("pin %p %s:%s, unexpected call.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
     /* not supposed to do anything in an output pin */
 
     return E_UNEXPECTED;
 }
 
-HRESULT WINAPI BaseOutputPinImpl_BeginFlush(IPin * iface)
+static HRESULT WINAPI source_BeginFlush(IPin *iface)
 {
-    TRACE("(%p)->()\n", iface);
+    struct strmbase_source *pin = impl_source_from_IPin(iface);
+
+    WARN("pin %p %s:%s, unexpected call.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
     /* not supposed to do anything in an output pin */
 
     return E_UNEXPECTED;
 }
 
-HRESULT WINAPI BaseOutputPinImpl_EndFlush(IPin * iface)
+static HRESULT WINAPI source_EndFlush(IPin *iface)
 {
-    TRACE("(%p)->()\n", iface);
+    struct strmbase_source *pin = impl_source_from_IPin(iface);
+
+    WARN("pin %p %s:%s, unexpected call.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
     /* not supposed to do anything in an output pin */
 
     return E_UNEXPECTED;
 }
 
+static HRESULT WINAPI source_NewSegment(IPin * iface, REFERENCE_TIME start, REFERENCE_TIME stop, double rate)
+{
+    struct strmbase_source *pin = impl_source_from_IPin(iface);
+
+    TRACE("pin %p %s:%s, start %s, stop %s, rate %.16e.\n", pin, debugstr_w(pin->pin.filter->name),
+            debugstr_w(pin->pin.name), debugstr_time(start), debugstr_time(stop), rate);
+
+    return S_OK;
+}
+
+static const IPinVtbl source_vtbl =
+{
+    pin_QueryInterface,
+    pin_AddRef,
+    pin_Release,
+    source_Connect,
+    source_ReceiveConnection,
+    source_Disconnect,
+    pin_ConnectedTo,
+    pin_ConnectionMediaType,
+    pin_QueryPinInfo,
+    pin_QueryDirection,
+    pin_QueryId,
+    pin_QueryAccept,
+    pin_EnumMediaTypes,
+    pin_QueryInternalConnections,
+    source_EndOfStream,
+    source_BeginFlush,
+    source_EndFlush,
+    source_NewSegment,
+};
+
 HRESULT WINAPI BaseOutputPinImpl_GetDeliveryBuffer(struct strmbase_source *This,
         IMediaSample **ppSample, REFERENCE_TIME *tStart, REFERENCE_TIME *tStop, DWORD dwFlags)
 {
@@ -595,11 +783,11 @@ HRESULT WINAPI BaseOutputPinImpl_AttemptConnection(struct strmbase_source *This,
     return hr;
 }
 
-void strmbase_source_init(struct strmbase_source *pin, const IPinVtbl *vtbl, struct strmbase_filter *filter,
+void strmbase_source_init(struct strmbase_source *pin, struct strmbase_filter *filter,
         const WCHAR *name, const struct strmbase_source_ops *func_table)
 {
     memset(pin, 0, sizeof(*pin));
-    pin->pin.IPin_iface.lpVtbl = vtbl;
+    pin->pin.IPin_iface.lpVtbl = &source_vtbl;
     pin->pin.filter = filter;
     pin->pin.dir = PINDIR_OUTPUT;
     lstrcpyW(pin->pin.name, name);
@@ -620,22 +808,30 @@ static struct strmbase_sink *impl_sink_from_IPin(IPin *iface)
     return CONTAINING_RECORD(iface, struct strmbase_sink, pin.IPin_iface);
 }
 
-HRESULT WINAPI BaseInputPinImpl_Connect(IPin *iface, IPin *pin, const AM_MEDIA_TYPE *pmt)
+static HRESULT WINAPI sink_Connect(IPin *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
 {
-    ERR("(%p)->(%p, %p) outgoing connection on an input pin!\n", iface, pin, pmt);
+    struct strmbase_sink *pin = impl_sink_from_IPin(iface);
+
+    WARN("pin %p %s:%s, peer %p, mt %p, unexpected call.\n", pin, debugstr_w(pin->pin.name),
+            debugstr_w(pin->pin.filter->name), peer, mt);
+
     return E_UNEXPECTED;
 }
 
 
-HRESULT WINAPI BaseInputPinImpl_ReceiveConnection(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI sink_ReceiveConnection(IPin *iface, IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
 {
     struct strmbase_sink *This = impl_sink_from_IPin(iface);
     PIN_DIRECTION pindirReceive;
     HRESULT hr = S_OK;
 
-    TRACE("(%p)->(%p, %p)\n", This, pReceivePin, pmt);
+    TRACE("pin %p %s:%s, peer %p, mt %p.\n", This, debugstr_w(This->pin.filter->name),
+            debugstr_w(This->pin.name), pReceivePin, pmt);
     strmbase_dump_media_type(pmt);
 
+    if (!pmt)
+        return E_POINTER;
+
     EnterCriticalSection(&This->pin.filter->csFilter);
     {
         if (This->pin.filter->state != State_Stopped)
@@ -663,6 +859,9 @@ HRESULT WINAPI BaseInputPinImpl_ReceiveConnection(IPin * iface, IPin * pReceiveP
             }
         }
 
+        if (SUCCEEDED(hr) && This->pFuncsTable->sink_connect)
+            hr = This->pFuncsTable->sink_connect(This, pReceivePin, pmt);
+
         if (SUCCEEDED(hr))
         {
             CopyMediaType(&This->pin.mt, pmt);
@@ -675,17 +874,48 @@ HRESULT WINAPI BaseInputPinImpl_ReceiveConnection(IPin * iface, IPin * pReceiveP
     return hr;
 }
 
+static HRESULT WINAPI sink_Disconnect(IPin *iface)
+{
+    struct strmbase_sink *pin = impl_sink_from_IPin(iface);
+    HRESULT hr;
+
+    TRACE("pin %p %s:%s.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
+
+    EnterCriticalSection(&pin->pin.filter->csFilter);
+
+    if (pin->pin.peer)
+    {
+        if (pin->pFuncsTable->sink_disconnect)
+            pin->pFuncsTable->sink_disconnect(pin);
+
+        IPin_Release(pin->pin.peer);
+        pin->pin.peer = NULL;
+        FreeMediaType(&pin->pin.mt);
+        memset(&pin->pin.mt, 0, sizeof(AM_MEDIA_TYPE));
+        hr = S_OK;
+    }
+    else
+        hr = S_FALSE;
+
+    LeaveCriticalSection(&pin->pin.filter->csFilter);
+
+    return hr;
+}
+
 static HRESULT deliver_endofstream(IPin* pin, LPVOID unused)
 {
     return IPin_EndOfStream( pin );
 }
 
-HRESULT WINAPI BaseInputPinImpl_EndOfStream(IPin * iface)
+static HRESULT WINAPI sink_EndOfStream(IPin *iface)
 {
     struct strmbase_sink *This = impl_sink_from_IPin(iface);
     HRESULT hr = S_OK;
 
-    TRACE("(%p)->()\n", This);
+    TRACE("pin %p %s:%s.\n", This, debugstr_w(This->pin.filter->name), debugstr_w(This->pin.name));
+
+    if (This->pFuncsTable->sink_eos)
+        return This->pFuncsTable->sink_eos(This);
 
     EnterCriticalSection(&This->pin.filter->csFilter);
     if (This->flushing)
@@ -702,17 +932,23 @@ static HRESULT deliver_beginflush(IPin* pin, LPVOID unused)
     return IPin_BeginFlush( pin );
 }
 
-HRESULT WINAPI BaseInputPinImpl_BeginFlush(IPin * iface)
+static HRESULT WINAPI sink_BeginFlush(IPin *iface)
 {
-    struct strmbase_sink *This = impl_sink_from_IPin(iface);
+    struct strmbase_sink *pin = impl_sink_from_IPin(iface);
     HRESULT hr;
-    TRACE("(%p) semi-stub\n", This);
 
-    EnterCriticalSection(&This->pin.filter->csFilter);
-    This->flushing = TRUE;
+    TRACE("pin %p %s:%s.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
-    hr = SendFurther(This, deliver_beginflush, NULL);
-    LeaveCriticalSection(&This->pin.filter->csFilter);
+    EnterCriticalSection(&pin->pin.filter->csFilter);
+
+    pin->flushing = TRUE;
+
+    if (pin->pFuncsTable->sink_begin_flush)
+        hr = pin->pFuncsTable->sink_begin_flush(pin);
+    else
+        hr = SendFurther(pin, deliver_beginflush, NULL);
+
+    LeaveCriticalSection(&pin->pin.filter->csFilter);
 
     return hr;
 }
@@ -722,17 +958,23 @@ static HRESULT deliver_endflush(IPin* pin, LPVOID unused)
     return IPin_EndFlush( pin );
 }
 
-HRESULT WINAPI BaseInputPinImpl_EndFlush(IPin * iface)
+static HRESULT WINAPI sink_EndFlush(IPin * iface)
 {
-    struct strmbase_sink *This = impl_sink_from_IPin(iface);
+    struct strmbase_sink *pin = impl_sink_from_IPin(iface);
     HRESULT hr;
-    TRACE("(%p)->()\n", This);
 
-    EnterCriticalSection(&This->pin.filter->csFilter);
-    This->flushing = FALSE;
+    TRACE("pin %p %s:%s.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
-    hr = SendFurther(This, deliver_endflush, NULL);
-    LeaveCriticalSection(&This->pin.filter->csFilter);
+    EnterCriticalSection(&pin->pin.filter->csFilter);
+
+    pin->flushing = FALSE;
+
+    if (pin->pFuncsTable->sink_end_flush)
+        hr = pin->pFuncsTable->sink_end_flush(pin);
+    else
+        hr = SendFurther(pin, deliver_endflush, NULL);
+
+    LeaveCriticalSection(&pin->pin.filter->csFilter);
 
     return hr;
 }
@@ -749,13 +991,16 @@ static HRESULT deliver_newsegment(IPin *pin, LPVOID data)
     return IPin_NewSegment(pin, args->tStart, args->tStop, args->rate);
 }
 
-HRESULT WINAPI BaseInputPinImpl_NewSegment(IPin * iface, REFERENCE_TIME start, REFERENCE_TIME stop, double rate)
+static HRESULT WINAPI sink_NewSegment(IPin *iface, REFERENCE_TIME start, REFERENCE_TIME stop, double rate)
 {
     struct strmbase_sink *pin = impl_sink_from_IPin(iface);
     newsegmentargs args;
 
-    TRACE("iface %p, start %s, stop %s, rate %.16e.\n",
-            iface, debugstr_time(start), debugstr_time(stop), rate);
+    TRACE("pin %p %s:%s, start %s, stop %s, rate %.16e.\n", pin, debugstr_w(pin->pin.filter->name),
+            debugstr_w(pin->pin.name), debugstr_time(start), debugstr_time(stop), rate);
+
+    if (pin->pFuncsTable->sink_new_segment)
+        return pin->pFuncsTable->sink_new_segment(pin, start, stop, rate);
 
     args.tStart = start;
     args.tStop = stop;
@@ -764,6 +1009,28 @@ HRESULT WINAPI BaseInputPinImpl_NewSegment(IPin * iface, REFERENCE_TIME start, R
     return SendFurther(pin, deliver_newsegment, &args);
 }
 
+static const IPinVtbl sink_vtbl =
+{
+    pin_QueryInterface,
+    pin_AddRef,
+    pin_Release,
+    sink_Connect,
+    sink_ReceiveConnection,
+    sink_Disconnect,
+    pin_ConnectedTo,
+    pin_ConnectionMediaType,
+    pin_QueryPinInfo,
+    pin_QueryDirection,
+    pin_QueryId,
+    pin_QueryAccept,
+    pin_EnumMediaTypes,
+    pin_QueryInternalConnections,
+    sink_EndOfStream,
+    sink_BeginFlush,
+    sink_EndFlush,
+    sink_NewSegment,
+};
+
 /*** IMemInputPin implementation ***/
 
 static inline struct strmbase_sink *impl_from_IMemInputPin(IMemInputPin *iface)
@@ -796,7 +1063,8 @@ static HRESULT WINAPI MemInputPin_GetAllocator(IMemInputPin * iface, IMemAllocat
 {
     struct strmbase_sink *This = impl_from_IMemInputPin(iface);
 
-    TRACE("(%p/%p)->(%p)\n", This, iface, ppAllocator);
+    TRACE("pin %p %s:%s, allocator %p.\n", This, debugstr_w(This->pin.filter->name),
+            debugstr_w(This->pin.name), ppAllocator);
 
     *ppAllocator = This->pAllocator;
     if (*ppAllocator)
@@ -809,7 +1077,8 @@ static HRESULT WINAPI MemInputPin_NotifyAllocator(IMemInputPin * iface, IMemAllo
 {
     struct strmbase_sink *This = impl_from_IMemInputPin(iface);
 
-    TRACE("(%p/%p)->(%p, %d)\n", This, iface, pAllocator, bReadOnly);
+    TRACE("pin %p %s:%s, allocator %p, read_only %d.\n", This, debugstr_w(This->pin.filter->name),
+            debugstr_w(This->pin.name), pAllocator, bReadOnly);
 
     if (bReadOnly)
         FIXME("Read only flag not handled yet!\n");
@@ -833,36 +1102,35 @@ static HRESULT WINAPI MemInputPin_NotifyAllocator(IMemInputPin * iface, IMemAllo
     return S_OK;
 }
 
-static HRESULT WINAPI MemInputPin_GetAllocatorRequirements(IMemInputPin * iface, ALLOCATOR_PROPERTIES * pProps)
+static HRESULT WINAPI MemInputPin_GetAllocatorRequirements(IMemInputPin *iface, ALLOCATOR_PROPERTIES *props)
 {
-    struct strmbase_sink *This = impl_from_IMemInputPin(iface);
+    struct strmbase_sink *pin = impl_from_IMemInputPin(iface);
 
-    TRACE("(%p/%p)->(%p)\n", This, iface, pProps);
+    TRACE("pin %p %s:%s, props %p.\n", pin, debugstr_w(pin->pin.filter->name),
+            debugstr_w(pin->pin.name), props);
 
     /* override this method if you have any specific requirements */
 
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI MemInputPin_Receive(IMemInputPin * iface, IMediaSample * pSample)
+static HRESULT WINAPI MemInputPin_Receive(IMemInputPin *iface, IMediaSample *sample)
 {
-    struct strmbase_sink *This = impl_from_IMemInputPin(iface);
+    struct strmbase_sink *pin = impl_from_IMemInputPin(iface);
     HRESULT hr = S_FALSE;
 
-    /* this trace commented out for performance reasons */
-    /*TRACE("(%p/%p)->(%p)\n", This, iface, pSample);*/
-    if (This->pFuncsTable->pfnReceive)
-        hr = This->pFuncsTable->pfnReceive(This, pSample);
+    TRACE("pin %p %s:%s, sample %p.\n", pin, debugstr_w(pin->pin.filter->name),
+            debugstr_w(pin->pin.name), sample);
+
+    if (pin->pFuncsTable->pfnReceive)
+        hr = pin->pFuncsTable->pfnReceive(pin, sample);
     return hr;
 }
 
 static HRESULT WINAPI MemInputPin_ReceiveMultiple(IMemInputPin * iface, IMediaSample ** pSamples, LONG nSamples, LONG *nSamplesProcessed)
 {
-    struct strmbase_sink *This = impl_from_IMemInputPin(iface);
     HRESULT hr = S_OK;
 
-    TRACE("(%p/%p)->(%p, %d, %p)\n", This, iface, pSamples, nSamples, nSamplesProcessed);
-
     for (*nSamplesProcessed = 0; *nSamplesProcessed < nSamples; (*nSamplesProcessed)++)
     {
         hr = IMemInputPin_Receive(iface, pSamples[*nSamplesProcessed]);
@@ -875,9 +1143,9 @@ static HRESULT WINAPI MemInputPin_ReceiveMultiple(IMemInputPin * iface, IMediaSa
 
 static HRESULT WINAPI MemInputPin_ReceiveCanBlock(IMemInputPin * iface)
 {
-    struct strmbase_sink *This = impl_from_IMemInputPin(iface);
+    struct strmbase_sink *pin = impl_from_IMemInputPin(iface);
 
-    TRACE("(%p/%p)->()\n", This, iface);
+    TRACE("pin %p %s:%s.\n", pin, debugstr_w(pin->pin.filter->name), debugstr_w(pin->pin.name));
 
     return S_OK;
 }
@@ -895,11 +1163,11 @@ static const IMemInputPinVtbl MemInputPin_Vtbl =
     MemInputPin_ReceiveCanBlock
 };
 
-void strmbase_sink_init(struct strmbase_sink *pin, const IPinVtbl *vtbl, struct strmbase_filter *filter,
+void strmbase_sink_init(struct strmbase_sink *pin, struct strmbase_filter *filter,
         const WCHAR *name, const struct strmbase_sink_ops *func_table, IMemAllocator *allocator)
 {
     memset(pin, 0, sizeof(*pin));
-    pin->pin.IPin_iface.lpVtbl = vtbl;
+    pin->pin.IPin_iface.lpVtbl = &sink_vtbl;
     pin->pin.filter = filter;
     pin->pin.dir = PINDIR_INPUT;
     lstrcpyW(pin->pin.name, name);
