@@ -26,9 +26,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(strmbase);
 static const WCHAR wcsInputPinName[] = {'I','n',0};
 static const WCHAR wcsOutputPinName[] = {'O','u','t',0};
 
-static const IPinVtbl TransformFilter_InputPin_Vtbl;
-static const IPinVtbl TransformFilter_OutputPin_Vtbl;
-
 static inline TransformFilter *impl_from_strmbase_filter(struct strmbase_filter *iface)
 {
     return CONTAINING_RECORD(iface, TransformFilter, filter);
@@ -41,14 +38,6 @@ static inline TransformFilter *impl_from_sink_IPin(IPin *iface)
 
 static HRESULT sink_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *pmt)
 {
-    TransformFilter *pTransform = impl_from_sink_IPin(&iface->IPin_iface);
-
-    TRACE("%p\n", iface);
-
-    if (pTransform->pFuncsTable->pfnCheckInputType)
-        return pTransform->pFuncsTable->pfnCheckInputType(pTransform, pmt);
-    /* Assume OK if there's no query method (the connection will fail if
-       needed) */
     return S_OK;
 }
 
@@ -82,12 +71,12 @@ static HRESULT WINAPI TransformFilter_Input_Receive(struct strmbase_sink *This,
         return S_FALSE;
     }
 
-    LeaveCriticalSection(&pTransform->csReceive);
     if (pTransform->pFuncsTable->pfnReceive)
         hr = pTransform->pFuncsTable->pfnReceive(pTransform, pInSample);
     else
         hr = S_FALSE;
 
+    LeaveCriticalSection(&pTransform->csReceive);
     return hr;
 }
 
@@ -213,12 +202,73 @@ static HRESULT sink_query_interface(struct strmbase_pin *iface, REFIID iid, void
     return S_OK;
 }
 
+static HRESULT sink_connect(struct strmbase_sink *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+
+    if (filter->pFuncsTable->transform_connect_sink)
+        return filter->pFuncsTable->transform_connect_sink(filter, mt);
+    return S_OK;
+}
+
+static void sink_disconnect(struct strmbase_sink *iface)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+
+    if (filter->pFuncsTable->pfnBreakConnect)
+        filter->pFuncsTable->pfnBreakConnect(filter, PINDIR_INPUT);
+}
+
+static HRESULT sink_eos(struct strmbase_sink *iface)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+
+    if (filter->source.pin.peer)
+        return IPin_EndOfStream(filter->source.pin.peer);
+    return VFW_E_NOT_CONNECTED;
+}
+
+static HRESULT sink_begin_flush(struct strmbase_sink *iface)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+    if (filter->source.pin.peer)
+        return IPin_BeginFlush(filter->source.pin.peer);
+    return S_OK;
+}
+
+static HRESULT sink_end_flush(struct strmbase_sink *iface)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+    HRESULT hr = S_OK;
+
+    if (filter->pFuncsTable->pfnEndFlush)
+        hr = filter->pFuncsTable->pfnEndFlush(filter);
+    if (SUCCEEDED(hr) && filter->source.pin.peer)
+        hr = IPin_EndFlush(filter->source.pin.peer);
+    return hr;
+}
+
+static HRESULT sink_new_segment(struct strmbase_sink *iface,
+        REFERENCE_TIME start, REFERENCE_TIME stop, double rate)
+{
+    TransformFilter *filter = impl_from_sink_IPin(&iface->pin.IPin_iface);
+    if (filter->source.pin.peer)
+        return IPin_NewSegment(filter->source.pin.peer, start, stop, rate);
+    return S_OK;
+}
+
 static const struct strmbase_sink_ops sink_ops =
 {
     .base.pin_query_accept = sink_query_accept,
     .base.pin_get_media_type = strmbase_pin_get_media_type,
     .base.pin_query_interface = sink_query_interface,
     .pfnReceive = TransformFilter_Input_Receive,
+    .sink_connect = sink_connect,
+    .sink_disconnect = sink_disconnect,
+    .sink_eos = sink_eos,
+    .sink_begin_flush = sink_begin_flush,
+    .sink_end_flush = sink_end_flush,
+    .sink_new_segment = sink_new_segment,
 };
 
 static HRESULT source_query_interface(struct strmbase_pin *iface, REFIID iid, void **out)
@@ -246,26 +296,6 @@ static const struct strmbase_source_ops source_ops =
     .pfnDecideAllocator = BaseOutputPinImpl_DecideAllocator,
 };
 
-
-static const IBaseFilterVtbl transform_vtbl =
-{
-    BaseFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    BaseFilterImpl_Stop,
-    BaseFilterImpl_Pause,
-    BaseFilterImpl_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
-};
-
 static TransformFilter *impl_from_source_IQualityControl(IQualityControl *iface)
 {
     return CONTAINING_RECORD(iface, TransformFilter, source_IQualityControl_iface);
@@ -346,7 +376,7 @@ static HRESULT strmbase_transform_init(IUnknown *outer, const CLSID *clsid,
     ISeekingPassThru *passthru;
     HRESULT hr;
 
-    strmbase_filter_init(&filter->filter, &transform_vtbl, outer, clsid, &filter_ops);
+    strmbase_filter_init(&filter->filter, outer, clsid, &filter_ops);
 
     InitializeCriticalSection(&filter->csReceive);
     filter->csReceive.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__": TransformFilter.csReceive");
@@ -355,11 +385,9 @@ static HRESULT strmbase_transform_init(IUnknown *outer, const CLSID *clsid,
     filter->pFuncsTable = func_table;
     ZeroMemory(&filter->pmt, sizeof(filter->pmt));
 
-    strmbase_sink_init(&filter->sink, &TransformFilter_InputPin_Vtbl, &filter->filter,
-            wcsInputPinName, &sink_ops, NULL);
+    strmbase_sink_init(&filter->sink, &filter->filter, wcsInputPinName, &sink_ops, NULL);
 
-    strmbase_source_init(&filter->source, &TransformFilter_OutputPin_Vtbl, &filter->filter,
-            wcsOutputPinName, &source_ops);
+    strmbase_source_init(&filter->source, &filter->filter, wcsOutputPinName, &source_ops);
     filter->source_IQualityControl_iface.lpVtbl = &source_qc_vtbl;
 
     filter->seekthru_unk = NULL;
@@ -407,143 +435,3 @@ HRESULT strmbase_transform_create(LONG filter_size, IUnknown *outer, const CLSID
     CoTaskMemFree(pTf);
     return E_FAIL;
 }
-
-static HRESULT WINAPI TransformFilter_InputPin_EndOfStream(IPin * iface)
-{
-    TransformFilter *filter = impl_from_sink_IPin(iface);
-
-    TRACE("iface %p.\n", iface);
-
-    if (filter->source.pin.peer)
-        return IPin_EndOfStream(filter->source.pin.peer);
-    return VFW_E_NOT_CONNECTED;
-}
-
-static HRESULT WINAPI TransformFilter_InputPin_ReceiveConnection(IPin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
-{
-    TransformFilter *pTransform = impl_from_sink_IPin(iface);
-    HRESULT hr = S_OK;
-
-    TRACE("(%p)->(%p, %p)\n", iface, pReceivePin, pmt);
-    strmbase_dump_media_type(pmt);
-
-    if (pTransform->pFuncsTable->pfnSetMediaType)
-        hr = pTransform->pFuncsTable->pfnSetMediaType(pTransform, PINDIR_INPUT, pmt);
-
-    if (SUCCEEDED(hr) && pTransform->pFuncsTable->pfnCompleteConnect)
-        hr = pTransform->pFuncsTable->pfnCompleteConnect(pTransform, PINDIR_INPUT, pReceivePin);
-
-    if (SUCCEEDED(hr))
-    {
-        hr = BaseInputPinImpl_ReceiveConnection(iface, pReceivePin, pmt);
-        if (FAILED(hr) && pTransform->pFuncsTable->pfnBreakConnect)
-            pTransform->pFuncsTable->pfnBreakConnect(pTransform, PINDIR_INPUT);
-    }
-
-    return hr;
-}
-
-static HRESULT WINAPI TransformFilter_InputPin_Disconnect(IPin * iface)
-{
-    TransformFilter *pTransform = impl_from_sink_IPin(iface);
-
-    TRACE("(%p)->()\n", iface);
-
-    if (pTransform->pFuncsTable->pfnBreakConnect)
-        pTransform->pFuncsTable->pfnBreakConnect(pTransform, PINDIR_INPUT);
-
-    return BasePinImpl_Disconnect(iface);
-}
-
-static HRESULT WINAPI TransformFilter_InputPin_BeginFlush(IPin * iface)
-{
-    TransformFilter *pTransform = impl_from_sink_IPin(iface);
-    HRESULT hr = S_OK;
-
-    TRACE("(%p)->()\n", iface);
-
-    EnterCriticalSection(&pTransform->filter.csFilter);
-    if (pTransform->pFuncsTable->pfnBeginFlush)
-        hr = pTransform->pFuncsTable->pfnBeginFlush(pTransform);
-    if (SUCCEEDED(hr))
-        hr = BaseInputPinImpl_BeginFlush(iface);
-    LeaveCriticalSection(&pTransform->filter.csFilter);
-    return hr;
-}
-
-static HRESULT WINAPI TransformFilter_InputPin_EndFlush(IPin * iface)
-{
-    TransformFilter *pTransform = impl_from_sink_IPin(iface);
-    HRESULT hr = S_OK;
-
-    TRACE("(%p)->()\n", iface);
-
-    EnterCriticalSection(&pTransform->filter.csFilter);
-    if (pTransform->pFuncsTable->pfnEndFlush)
-        hr = pTransform->pFuncsTable->pfnEndFlush(pTransform);
-    if (SUCCEEDED(hr))
-        hr = BaseInputPinImpl_EndFlush(iface);
-    LeaveCriticalSection(&pTransform->filter.csFilter);
-    return hr;
-}
-
-static HRESULT WINAPI TransformFilter_InputPin_NewSegment(IPin * iface, REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
-{
-    TransformFilter *pTransform = impl_from_sink_IPin(iface);
-    HRESULT hr = S_OK;
-
-    TRACE("iface %p, start %s, stop %s, rate %.16e.\n",
-            iface, debugstr_time(tStart), debugstr_time(tStop), dRate);
-
-    EnterCriticalSection(&pTransform->filter.csFilter);
-    if (pTransform->pFuncsTable->pfnNewSegment)
-        hr = pTransform->pFuncsTable->pfnNewSegment(pTransform, tStart, tStop, dRate);
-    if (SUCCEEDED(hr))
-        hr = BaseInputPinImpl_NewSegment(iface, tStart, tStop, dRate);
-    LeaveCriticalSection(&pTransform->filter.csFilter);
-    return hr;
-}
-
-static const IPinVtbl TransformFilter_InputPin_Vtbl =
-{
-    BasePinImpl_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseInputPinImpl_Connect,
-    TransformFilter_InputPin_ReceiveConnection,
-    TransformFilter_InputPin_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    TransformFilter_InputPin_EndOfStream,
-    TransformFilter_InputPin_BeginFlush,
-    TransformFilter_InputPin_EndFlush,
-    TransformFilter_InputPin_NewSegment
-};
-
-static const IPinVtbl TransformFilter_OutputPin_Vtbl =
-{
-    BasePinImpl_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseOutputPinImpl_Connect,
-    BaseOutputPinImpl_ReceiveConnection,
-    BaseOutputPinImpl_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    BaseOutputPinImpl_EndOfStream,
-    BaseOutputPinImpl_BeginFlush,
-    BaseOutputPinImpl_EndFlush,
-    BasePinImpl_NewSegment
-};
