@@ -233,6 +233,7 @@ static HRESULT adapter_vk_create_device(struct wined3d *wined3d, const struct wi
 
     device_vk->vk_device = vk_device;
     VK_CALL(vkGetDeviceQueue(vk_device, queue_family_index, 0, &device_vk->vk_queue));
+    device_vk->vk_queue_family_index = queue_family_index;
 
     device_vk->vk_info = *vk_info;
 #define LOAD_DEVICE_PFN(name) \
@@ -448,46 +449,107 @@ static HRESULT adapter_vk_init_3d(struct wined3d_device *device)
     return WINED3D_OK;
 }
 
-static void adapter_vk_uninit_3d(struct wined3d_device *device)
+static void adapter_vk_uninit_3d_cs(void *object)
 {
+    struct wined3d_device_vk *device_vk = object;
     struct wined3d_context_vk *context_vk;
+    struct wined3d_device *device;
     struct wined3d_shader *shader;
 
-    TRACE("device %p.\n", device);
+    TRACE("device_vk %p.\n", device_vk);
 
-    context_vk = &wined3d_device_vk(device)->context_vk;
+    context_vk = &device_vk->context_vk;
+    device = &device_vk->d;
 
     LIST_FOR_EACH_ENTRY(shader, &device->shaders, struct wined3d_shader, shader_list_entry)
     {
         device->shader_backend->shader_destroy(shader);
     }
 
-    wined3d_device_destroy_default_samplers(device, &context_vk->c);
-
     device->blitter->ops->blitter_destroy(device->blitter, NULL);
+    device->shader_backend->shader_free_private(device, &context_vk->c);
+    wined3d_device_destroy_default_samplers(device, &context_vk->c);
+}
+
+static void adapter_vk_uninit_3d(struct wined3d_device *device)
+{
+    struct wined3d_context_vk *context_vk;
+    struct wined3d_device_vk *device_vk;
+
+    TRACE("device %p.\n", device);
+
+    device_vk = wined3d_device_vk(device);
+    context_vk = &device_vk->context_vk;
+
+    wined3d_cs_destroy_object(device->cs, adapter_vk_uninit_3d_cs, device_vk);
+    wined3d_cs_finish(device->cs, WINED3D_CS_QUEUE_DEFAULT);
 
     device_context_remove(device, &context_vk->c);
-    device->shader_backend->shader_free_private(device, NULL);
     wined3d_context_vk_cleanup(context_vk);
 }
 
 static void *adapter_vk_map_bo_address(struct wined3d_context *context,
         const struct wined3d_bo_address *data, size_t size, uint32_t bind_flags, uint32_t map_flags)
 {
-    if (data->buffer_object)
+    struct wined3d_context_vk *context_vk = wined3d_context_vk(context);
+    const struct wined3d_vk_info *vk_info;
+    struct wined3d_device_vk *device_vk;
+    VkCommandBuffer vk_command_buffer;
+    VkBufferMemoryBarrier vk_barrier;
+    struct wined3d_bo_vk *bo;
+    void *map_ptr;
+    VkResult vr;
+
+    if (!(bo = (struct wined3d_bo_vk *)data->buffer_object))
+        return data->addr;
+
+    vk_info = context_vk->vk_info;
+    device_vk = wined3d_device_vk(context->device);
+
+    if (!(vk_command_buffer = wined3d_context_vk_get_command_buffer(context_vk)))
     {
-        ERR("Unsupported buffer object %#lx.\n", data->buffer_object);
+        ERR("Failed to get command buffer.\n");
         return NULL;
     }
 
-    return data->addr;
+    vk_barrier.sType = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
+    vk_barrier.pNext = NULL;
+    vk_barrier.srcAccessMask = vk_access_mask_from_bind_flags(bind_flags);
+    vk_barrier.dstAccessMask = VK_ACCESS_HOST_READ_BIT;
+    vk_barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+    vk_barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
+    vk_barrier.buffer = bo->vk_buffer;
+    vk_barrier.offset = (uintptr_t)data->addr;
+    vk_barrier.size = size;
+    VK_CALL(vkCmdPipelineBarrier(vk_command_buffer, VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
+            VK_PIPELINE_STAGE_HOST_BIT, 0, 0, NULL, 1, &vk_barrier, 0, NULL));
+
+    wined3d_context_vk_submit_command_buffer(context_vk);
+    wined3d_context_vk_wait_command_buffer(context_vk, context_vk->current_command_buffer.id - 1);
+
+    if ((vr = VK_CALL(vkMapMemory(device_vk->vk_device, bo->vk_memory,
+            (uintptr_t)data->addr, size, 0, &map_ptr))) < 0)
+    {
+        ERR("Failed to map buffer, vr %s.\n", wined3d_debug_vkresult(vr));
+        return NULL;
+    }
+
+    return map_ptr;
 }
 
 static void adapter_vk_unmap_bo_address(struct wined3d_context *context, const struct wined3d_bo_address *data,
         uint32_t bind_flags, unsigned int range_count, const struct wined3d_map_range *ranges)
 {
-    if (data->buffer_object)
-        ERR("Unsupported buffer object %#lx.\n", data->buffer_object);
+    const struct wined3d_vk_info *vk_info;
+    struct wined3d_device_vk *device_vk;
+    struct wined3d_bo_vk *bo;
+
+    if (!(bo = (struct wined3d_bo_vk *)data->buffer_object))
+        return;
+
+    vk_info = wined3d_context_vk(context)->vk_info;
+    device_vk = wined3d_device_vk(context->device);
+    VK_CALL(vkUnmapMemory(device_vk->vk_device, bo->vk_memory));
 }
 
 static void adapter_vk_copy_bo_address(struct wined3d_context *context,
@@ -539,6 +601,23 @@ static void adapter_vk_destroy_swapchain(struct wined3d_swapchain *swapchain)
     heap_free(swapchain);
 }
 
+unsigned int wined3d_adapter_vk_get_memory_type_index(const struct wined3d_adapter_vk *adapter_vk,
+        uint32_t memory_type_mask, VkMemoryPropertyFlags flags)
+{
+    const VkPhysicalDeviceMemoryProperties *memory_info = &adapter_vk->memory_properties;
+    unsigned int i;
+
+    for (i = 0; i < memory_info->memoryTypeCount; ++i)
+    {
+        if (!(memory_type_mask & (1u << i)))
+            continue;
+        if ((memory_info->memoryTypes[i].propertyFlags & flags) == flags)
+            return i;
+    }
+
+    return ~0u;
+}
+
 static HRESULT adapter_vk_create_buffer(struct wined3d_device *device,
         const struct wined3d_buffer_desc *desc, const struct wined3d_sub_resource_data *data,
         void *parent, const struct wined3d_parent_ops *parent_ops, struct wined3d_buffer **buffer)
@@ -1084,9 +1163,10 @@ static enum wined3d_display_driver guess_display_driver(enum wined3d_pci_vendor
     }
 }
 
-static void adapter_vk_init_driver_info(struct wined3d_adapter *adapter,
-        const VkPhysicalDeviceProperties *properties, const VkPhysicalDeviceMemoryProperties *memory_properties)
+static void adapter_vk_init_driver_info(struct wined3d_adapter_vk *adapter_vk,
+        const VkPhysicalDeviceProperties *properties)
 {
+    const VkPhysicalDeviceMemoryProperties *memory_properties = &adapter_vk->memory_properties;
     const struct wined3d_gpu_description *gpu_description;
     struct wined3d_gpu_description description;
     UINT64 vram_bytes, sysmem_bytes;
@@ -1128,7 +1208,7 @@ static void adapter_vk_init_driver_info(struct wined3d_adapter *adapter,
         gpu_description = &description;
     }
 
-    wined3d_driver_info_init(&adapter->driver_info, gpu_description, vram_bytes, sysmem_bytes);
+    wined3d_driver_info_init(&adapter_vk->a.driver_info, gpu_description, vram_bytes, sysmem_bytes);
 }
 
 static void wined3d_adapter_vk_init_d3d_info(struct wined3d_adapter *adapter, uint32_t wined3d_creation_flags)
@@ -1146,7 +1226,6 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
         unsigned int ordinal, unsigned int wined3d_creation_flags)
 {
     struct wined3d_vk_info *vk_info = &adapter_vk->vk_info;
-    VkPhysicalDeviceMemoryProperties memory_properties;
     struct wined3d_adapter *adapter = &adapter_vk->a;
     VkPhysicalDeviceIDProperties id_properties;
     VkPhysicalDeviceProperties2 properties2;
@@ -1177,9 +1256,9 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
         VK_CALL(vkGetPhysicalDeviceProperties(adapter_vk->physical_device, &properties2.properties));
     adapter_vk->device_limits = properties2.properties.limits;
 
-    VK_CALL(vkGetPhysicalDeviceMemoryProperties(adapter_vk->physical_device, &memory_properties));
+    VK_CALL(vkGetPhysicalDeviceMemoryProperties(adapter_vk->physical_device, &adapter_vk->memory_properties));
 
-    adapter_vk_init_driver_info(adapter, &properties2.properties, &memory_properties);
+    adapter_vk_init_driver_info(adapter_vk, &properties2.properties);
     adapter->vram_bytes_used = 0;
     TRACE("Emulating 0x%s bytes of video ram.\n", wine_dbgstr_longlong(adapter->driver_info.vram_bytes));
 
